//Time-stamp: <2004-06-01 13:48:34 mary>
/*  
 *
 
Initiation to the simulator
 - use of OpenGL display
 - Gestion of keyboar and mouse
 - definition of an environment
 - definition of a basic controller
 - using of two robots
 
 *
 */

//Classics headers
//#include <iostream>

// To use time to initialise Random number generator
#include <time.h>

//Environment, Robot,  Word and Controllers headers
#include "../src/heart/simulator.h"


//We prepare a function to manage keys hitted during simulation
// c is the char received
// (x,y) the position of the mouse in the window
void my_key_func(unsigned char c, int x, int y)
{

  cout<<"You hitted --"<<c<<"-- with mouse at :"<<x<<","<<y<<endl;
  
}



// We are going to write a basic controller for khepera.
// The purpose of a controller is to receive as an argument a robot
// and to fill the struc decision of this robot
// For a khepera the struc is :
//
// struct decision_khepera
//  {
//    Real left_motor; //value_between 0.0 and 1.0, between 0.0 and 0.5, the wheel goes backward and between 0.5 and 1.0 the wheel goes forward
//    Real right_motor; //value_between 0.0 and 1.0, between 0.0 and 0.5, the wheel goes backward and between 0.5 and 1.0 the wheel goes forward
//    bool gripper_take;
//    bool gripper_release;
//  }
//
// You can name the function as you want
void simple_khepera_controller (robot & _myrobot) 
	 
{
  // Firstly we have to indicate that the robot received is a khepera
  // In a magic world you should have a test to detect the type of the robot
  // and have a case for each type possible. So when you use this controller
  // beware to give it a kephera (or a derivate classe)
#ifdef _WIN32
  khepera * myrobot =(static_cast<khepera *>(& _myrobot));
#else
  khepera * myrobot =(dynamic_cast<khepera *>(& _myrobot));
#endif
  
  if (myrobot->crash_obstacle) {cout<<"crash on obstacle"<<endl; }
  if (myrobot->crash_other_robot) {cout<<"crash on robot"<<endl; }
  if (myrobot->crash_obstacle_unfixed) {cout<<"crash on an unfixed object"<<endl; }
  if (myrobot->on_food)  {
	//Detect if robot is on food (food allow to eat)
	if (myrobot->eat()) {cout<<"miam-miam"<<endl;}
  }
  
  if ((myrobot->IR(2) < 20) && (myrobot->IR(3) < 20) &&!(myrobot->crash()))  //If the front sensors are not activated and no choc
		{
		  myrobot->decision.left_motor=1.0; 
		  myrobot->decision.right_motor=1.0; // the robot goes forward at maximal speed 
		}
  
	  else  //If the front sensors are activated
		{
		  myrobot->decision.left_motor=1.0; 
		  myrobot->decision.right_motor=0.0; // the robot turn on the right at maximal speed
		}
};

extern world world_GL;  // this is the world used by the main function and the main Display function for openGL


int main (int argc, char *argv[]) // int argc, char *argv[ ], char *envp[ ] )
{
  srand(time(NULL)); //random generator initialisation

  // We have to fill the world (world_GL is the displayed world)
  // We have to create an environment (for obstacles) an robots 
  
  //first of all : we think the name of function is expressive enough	 
  if (GL_world_initialisation(argc,argv)==1) {return 1; }//error with OpenGL

  //We register our callback function for keys management (Optionnal)
  Register_my_keyfunc(my_key_func);
  
  
  //secondly, we add the parameters for the environment and for the robots
  try	
	{
	  const int WIDTH  = 500;
	  const int LENGTH = 500;
	  
	  
      environment env0(WIDTH,LENGTH);// The two numbers are the size of the environment (in mm)

	  // We make the walls (give a different name for each one)
	  // The last value is the color (optionnal ; default is 255)
	  // 255: best shown
	  // 0 : the robot is blind to this color
      env0.add_wall("wall0",0.0,0.0,0.0,LENGTH,unsigned(255));
      env0.add_wall("wall1",0.0,0.0,WIDTH,0.0,unsigned(255));
      env0.add_wall("wall2",0.0,LENGTH,WIDTH,LENGTH,unsigned(255));
      env0.add_wall("wall3",WIDTH,0.0,WIDTH,LENGTH,unsigned(255));
      
	  // save way to add two obstacles (there is a method add_cylinder too)
      env0.add_cylinder_unfixed("cylinder_unfixed0",200.0,150.0,25.0,unsigned(255)); //this obstacle is unfixed (can move)
	  env0.add_bloc("bloc1",400.0,400.0,25.0,25.0,50.0,0.0,unsigned(255));

	  // We add food in the environment (there is an optionnal third parameter
	  // for the size and a fourth for the color.
	  // Actually modification of color doesn't affect display but is active
	  // for IR sensors and 1D cams (see tutorial4 for cams)
	  env0.add_food(100,300);
	  env0.add_food(100,200);
	  
	  
	  // You can make a save of your environment :
	  // env0.save("test_env_save.sav");
	  //
	  // And you can load it using this constructor
	  // environment env1("test_env_sav.sav");


	  // Now we create a controller using the function previously defined
	  Controller *  ctrl=new Controller (simple_khepera_controller);

	  // You can declare the controller you want to use and give them a name
	  // This is optionnal in a normal use (except if you plan to use EO)
	  all_the_controllers.add_a_controller("controller_khepera_simple",ctrl);
	  
	  //Ok, now we create two robots of type khepera (if you don't want to use all_the_controllers, just replace it by ctrl)
	  khepera myfirstkhepera ("first",all_the_controllers("controller_khepera_simple"),100.0,100.0,M_PI/2); // This robot have a fixed start and direction (direction is given in radian M_PI/2 is North)
	  khepera mysecondkhepera ("second",all_the_controllers("controller_khepera_simple")); // This robot have a random start


	  // Now we set the environment of the world
	  world_GL.set_environment(env0);
	  // And we add the robots 
	  world_GL.add_robot(myfirstkhepera);
	  world_GL.add_robot(mysecondkhepera); // you should always begin by adding the robots with a fixed start (there is no verification when a robot with fixed start is added, so...)

	 
	  
	  // main loop (only if you whish to use OPENG_GL display)
	  // See Glut Documentation (you will nevet get back from this function)
	  glutMainLoop();
	  
	  return 0;
		
	}
  
  // optionnal gestion of exceptions may help you...
  catch (error & _error)
	
	{
	  _error.what();
	}
  
  return 0;
  
  }
  



