//Time-stamp: <2003-11-04 18:53:40 mary>
/*
 *
 
Using lights and passive IR detection
 
 *
 */

//Classics headers
#include <iostream>

//Environment, Robot,  Word and Controllers headers
#include "../src/heart/simulator.h"

// To use time to initialise Random number generator
#include <time.h>


// You can name the function as you want
void khepera_controller_with_passive_IR (robot & _myrobot) 
	 
{
  // Firstly we have to indicate that the robot received is a khepera
#ifdef _WIN32
  khepera * myrobot =(static_cast<khepera *>(& _myrobot));
#else
  khepera * myrobot =(dynamic_cast<khepera *>(& _myrobot));
#endif
  
  if (myrobot->crash_obstacle) {cout<<"crash on obstacle"<<endl; }
  if (myrobot->crash_other_robot) {cout<<"crash on robot"<<endl; }
  if (myrobot->crash_obstacle_unfixed) {cout<<"crash on an unfixed object"<<endl; }
  if (myrobot->on_food)  {
	//Detect if robot is on food (food allow to eat)
	if (myrobot->eat()) {cout<<"miam-miam"<<endl;}
  }


  // search where is the light the capteur gives low values when it detects
  // lights (curious but as in a real khepera)
  int i,best=0;
  Real value=myrobot->IR_passive(0);

  for (i=0;i<8;i++)
	{
	  	  if ((myrobot->IR_passive(i))<value)
		{
		  //look the value of IR in mode passive
		  value=myrobot->IR_passive(i);
		  best=i;
		  }
	}

  // take an internal decision
  bool i_want_to_go_right=((best==4)||(best==6)||(best==5));
  bool i_want_to_go_left =((best==0)||(best==7)||(best==1));
  bool i_want_to_go_ahead=((best==2)||(best==3));
  
  
  // try to apply it
  if (i_want_to_go_ahead) {
	
  if ((myrobot->IR(2) < 20) && (myrobot->IR(3) < 20) &&!(myrobot->crash()))  //If the front sensors are not activated and no choc
		{
		  myrobot->decision.left_motor=1.0; 
		  myrobot->decision.right_motor=1.0; // the robot goes forward at maximal speed 
		}
  
	  else  //If the front sensors are activated
		{
		  
		  if ((myrobot->IR_passive(4))<(myrobot->IR_passive(1)))
			{
			  // if the light is best shown on the right
			  myrobot->decision.left_motor=1.0; 
			  myrobot->decision.right_motor=0.0; // the robot turn on the right at maximal speed
			} else {
			  myrobot->decision.left_motor=0.0; 
			  myrobot->decision.right_motor=1.0;
			}
		  
		  
		  
		}
  }

  if (i_want_to_go_left)
	{
	  myrobot->decision.left_motor=0.0; 
	  myrobot->decision.right_motor=1.0;
	}

  
  if (i_want_to_go_right)
	{
	  myrobot->decision.left_motor=1.0; 
	  myrobot->decision.right_motor=0.0;
	}
  
};

extern world world_GL;  // this is the world used by the main function and the main Display function for openGL


int main (int argc, char *argv[]) // int argc, char *argv[ ], char *envp[ ] )
{
  srand(time(NULL)); //random generator initialisation
  if (GL_world_initialisation(argc,argv)==1) {return 1; }//error with OpenGL

  try	
	{
	  const int WIDTH  = 500;
	  const int LENGTH = 500;
	  
	  
      environment env0(WIDTH,LENGTH);// The two numbers are the size of the environment (in mm)


      env0.add_wall("wall0",0.0,0.0,0.0,LENGTH,unsigned(255));
      env0.add_wall("wall1",0.0,0.0,WIDTH,0.0,unsigned(255));
      env0.add_wall("wall2",0.0,LENGTH,WIDTH,LENGTH,unsigned(255));
      env0.add_wall("wall3",WIDTH,0.0,WIDTH,LENGTH,unsigned(255));
      

      env0.add_cylinder_unfixed("cylinder_unfixed0",200.0,150.0,25.0,unsigned(255)); 
	  env0.add_bloc("bloc1",400.0,400.0,100.0,0.0,0.0,100.0,unsigned(255));


	  // We now add a light in the environment (we must have a light to have
	  // interesting using of passive IRs)
	  // the two first values are the coods of the light
	  // the last one is the power (the max value the light
	  // can give to an IR sensor, you can give value above 1024 but
	  //  a normal khepera have a treshold of perception of 1024)
	  // To modify the length of perception see global.h : SIZE_OF_LIGHTS
	  //
	  // BEWARE ligths are masked by other robots and obstacles
	  // So when a bot have found the light it can obstruct the other
	  //
	  env0.add_light("light1",100,100,1024);
	  env0.add_light("light2",100,400,1024);
	  

	  Controller *  ctrl=new Controller (khepera_controller_with_passive_IR);

	  all_the_controllers.add_a_controller("controller_khepera_passive_IR",ctrl);
	  
	  //Ok, now we create two robots of type khepera (if you don't want to use all_the_controllers, just replace it by ctrl)
	  khepera myfirstkhepera ("first",all_the_controllers("controller_khepera_passive_IR")); // This robot have a random start
	  khepera mysecondkhepera ("second",all_the_controllers("controller_khepera_passive_IR")); // This robot have a random start


	  // Now we set the environment of the world
	  world_GL.set_environment(env0);
	  // And we add the robots 
	  world_GL.add_robot(myfirstkhepera);
	  world_GL.add_robot(mysecondkhepera); 

	 
	  // Goo !!!!!!
	  glutMainLoop();
	  
	  return 0;
		
	}
  
  // optionnal gestion of exceptions may help you...
  catch (error & _error)
	
	{
	  _error.what();
	}
  
  return 0;
  
  }
  



