//Time-stamp: <2003-10-29 13:13:48 mary>
/*  
 *
 
Using of cameras
 -1D cams
 -2D cams
 
 *
 */

//Classics headers
#include <iostream>

//Environment, Robot,  Word and Controllers headers
#include "../src/heart/simulator.h"

// To use time to initialise Random number generator
#include <time.h>


void khepera_controller_with2D_cam (robot & _myrobot) 
	 
{
  // Firstly we have to indicate that the robot received is a khepera
  // In a magic world you should have a test to detect the type of the robot
  // and have a case for each type possible. So when you use this controller
  // beware to give it a kephera (or a derivate classe)
#ifdef _WIN32
  khepera * myrobot =(static_cast<khepera *>(& _myrobot));
#else
  khepera * myrobot =(dynamic_cast<khepera *>(& _myrobot));
#endif

  // To have the RGB values of the pixel x,y of the cam
  // (0,0) is at the bottom left (i'd prefer upper left but openGL
  // glReadPixel was already coded ;-) )
  //
  // calc the offset : O=(320*y+x)*3
  //
  // the R composant is at O
  // the G composant is at O+1
  // the B composant is at O+2
  //
  // Rq: by default x E [0,319], y E [0,199]
  // To this point no control over your access so... 

  // In RGB yellow is given by  high R and G and low B
  // So we try to detect yellow (food) in the camera's picture
  
  const int divide=8;
  
  bool internal_picture[KHEPERA_CAM_2D_X/divide+1][KHEPERA_CAM_2D_Y/divide+1];

  int i,j;
  // I know it would be more efficient to avoid divisions and it isn't very
  // difficult here but i think this it is easiest to undersand as is
  for (i=0;i<KHEPERA_CAM_2D_X;i+=divide)
	{
	  for (j=0;j<KHEPERA_CAM_2D_Y;j+=divide)
		{
		  if (
			  (int)(myrobot->camera2D())[(j*KHEPERA_CAM_2D_X+i)*3]>80 &&
			  (int)(myrobot->camera2D())[(j*KHEPERA_CAM_2D_X+i)*3+1]>70 &&
			  (int)(myrobot->camera2D())[(j*KHEPERA_CAM_2D_X+i)*3+2]<100
			  )
			{
			  internal_picture[i/divide][j/divide]=true; //the point is yellow
			  // cout<<"1";
			  
			} else {
			  internal_picture[i/divide][j/divide]=false; //other color
			  //cout<<"0";
			}
		}
	  // cout<<endl;
	  	}
  //cout<<endl;


  
  //Ok now we transform this 2D internal representation in a 1D representation
  bool internal_picture1D[KHEPERA_CAM_2D_X/divide+1];
  for (i=0;i<KHEPERA_CAM_2D_X/divide;i++)
	{
	  bool buffer=false;
	  for (j=0;j<KHEPERA_CAM_2D_Y/divide;j++)
		{
		  buffer=(buffer||internal_picture[i][j]);
		}
	  internal_picture1D[i]=buffer;
	  
	}

  //You want to see the 1D representation uncomment this :
  /*
  for (i=0;i<KHEPERA_CAM_2D_X/divide;i++)
	{
	  if (internal_picture1D[i]) {
		cout<<"1";
	  } else {
		cout<<"0";
	  }
	  
	}
  cout<<endl;
  */

  int look_length=(KHEPERA_CAM_2D_X/(divide*3));
  int count_left=0,count_right=0,count_ahead=0;

  for (i=0;i<look_length;i++)
	{
	  if (internal_picture1D[i]) {count_left++;}
	  if (internal_picture1D[KHEPERA_CAM_2D_X/divide-i]) {count_right++;}
	  if (internal_picture1D[KHEPERA_CAM_2D_X/(divide*2)-look_length/2+i]) {count_ahead+=2;}
	}


  if (myrobot->on_food)  {
	//Detect if robot is on food (food allow to eat)
	if (myrobot->eat()) {cout<<"miam-miam"<<endl;}
  }


  if ((count_ahead>=count_left)&&(count_ahead>=count_right))
	{
	  myrobot->decision.left_motor=1.0; 
	  myrobot->decision.right_motor=1.0; // the robot goes forward at maximal speed 
	}
  if ((count_right>=count_left)&&(count_right>count_ahead))
	{
	  myrobot->decision.left_motor=1.0; 
	  myrobot->decision.right_motor=0.0; // the robot turn on the right at maximal speed
	
	}
  if ((count_left>count_right)&&(count_left>count_ahead))
	{
	  myrobot->decision.left_motor=0.0; 
	  myrobot->decision.right_motor=1.0; // the robot turn on the left at maximal speed
	}
  
  
  if ((myrobot->crash())||((count_ahead==0)&&(count_right==0)&&(count_left==0)))
	{
	  myrobot->decision.left_motor=1.0; 
	  myrobot->decision.right_motor=0.0;
	}
  
};



//Now a controller which uses a 1D cam
void khepera_controller_with1D_cam (robot & _myrobot) 
{
  
#ifdef _WIN32
  khepera * myrobot =(static_cast<khepera *>(& _myrobot));
#else
  khepera * myrobot =(dynamic_cast<khepera *>(& _myrobot));
#endif

  
  int i;
	for (i=0;i<KHEPERA_CAM_1D_X*3;i++){
	  
	  cout<<myrobot->camera1D()[i];
	  
		}
  cout<<endl;
  
  
};





extern world world_GL;  // this is the world used by the main function and the main Display function for openGL


int main (int argc, char *argv[]) // int argc, char *argv[ ], char *envp[ ] )
{
  srand(time(NULL)); //random generator initialisation
  if (GL_world_initialisation(argc,argv)==1) {return 1; }//error with OpenGL
  

  try	
	{
	  const int WIDTH  = 500;
	  const int LENGTH = 500;
	  
	  
      environment env0(WIDTH,LENGTH);// The two numbers are the size of the environment (in mm)


      env0.add_wall("wall0",0.0,0.0,0.0,LENGTH,unsigned(255));
      env0.add_wall("wall1",0.0,0.0,WIDTH,0.0,unsigned(255));
      env0.add_wall("wall2",0.0,LENGTH,WIDTH,LENGTH,unsigned(255));
      env0.add_wall("wall3",WIDTH,0.0,WIDTH,LENGTH,unsigned(255));
      

      //env0.add_cylinder_unfixed("cylinder_unfixed0",200.0,150.0,25.0,unsigned(255));
	  //env0.add_bloc("bloc1",400.0,400.0,100.0,0.0,0.0,100.0,unsigned(255));

	  // We add food in the environment (there is an optionnal third parameter
	  // Which is the radius of the food )
	  for (int f=0;f<30;f++)
		{
		  env0.add_food(Math::UnitRandom()*(WIDTH-20)+10,
						Math::UnitRandom()*(LENGTH-20)+10
						);
		}
	  
	  
	 

	  // Now we create a controller using the function previously defined
	  Controller *  ctrl2D=new Controller (khepera_controller_with2D_cam);
	  Controller *  ctrl1D=new Controller (khepera_controller_with1D_cam);

	  // You can declare the controller you want to use and give them a name
	  // This is optionnal in a normal use (except if you plan to use EO)
	  all_the_controllers.add_a_controller("controller_cam2D",ctrl2D);
	  all_the_controllers.add_a_controller("controller_cam1D",ctrl1D);
	  

	  
	  //Ok, now we create two robots of type khepera 
	  khepera myfirstkhepera ("1D",all_the_controllers("controller_cam1D")); 
	  khepera mysecondkhepera ("2D",all_the_controllers("controller_cam2D")); 

	  //Do not forget to add cams to the robots
	  myfirstkhepera.add_1D_cam();
	  mysecondkhepera.add_2D_cam();
	  

	  // Now we set the environment of the world
	  world_GL.set_environment(env0);
	  // And we add the robots 
	  //world_GL.add_robot(myfirstkhepera);
	     world_GL.add_robot(mysecondkhepera); 
	 
	  
	  //Go
	  glutMainLoop();
	  
	  return 0;
		
	}
  
  // optionnal gestion of exceptions may help you...
  catch (error & _error)
	
	{
	  _error.what();
	}
  
  return 0;
  
  }
  



