#include "GeomFormCollisionDetection.h"
//#include <iostream>

using namespace Geom;

//----------------------------------------------------------------------------
bool Geom::GeomFormCollisionDetection ( Form& _form1,  Form& _form2)
{
  //the 4 "if" are used for fast detection collision using a bounding box
  
  if (_form1.x_max<_form2.x_min) return false;
  if (_form1.y_max<_form2.y_min) return false;
  if (_form2.x_max<_form1.x_min) return false;
  if (_form2.y_max<_form1.y_min) return false;
  
  

  
  
  Real w[2];
  Vector2 wcir[2];
  int control;
  string typ1,typ2;

    
    
  for (vector<PrimitivForm *>::iterator it1=_form1.forms.begin();it1 != _form1.forms.end();it1++) 
	{
	  
	  for (vector<PrimitivForm *>::iterator it2=_form2.forms.begin();it2 != _form2.forms.end();it2++) 
		{

		  typ1=(*it1)->type();
		  typ2=(*it2)->type();

#ifdef _WIN32

		  //-----------------------------------------------------------
		  //               Same objets intersections
		  //-----------------------------------------------------------
		  if ((typ1=="Circle") && (typ2=="Circle")) 
			{
			  if ( FindIntersection(*(static_cast<Circle2 *>(*it1)),
									*(static_cast<Circle2 *>(*it2)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Arc") && (typ2=="Arc")) 
			{
			  if ( FindIntersection(*(static_cast<Arc2 *>(*it1)),
									*(static_cast<Arc2 *>(*it2)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Segment") && (typ2=="Segment")) 
			{
			  if ( FindIntersection(*(static_cast<Segment2 *>(*it1)),
									*(static_cast<Segment2 *>(*it2)),
									control,w))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Line") && (typ2=="Line")) 
			{
			  if ( FindIntersection(*(static_cast<Line2 *>(*it1)),
									*(static_cast<Line2 *>(*it2)),
									control,w))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Ray") && (typ2=="Ray")) 
			{
			  if ( FindIntersection(*(static_cast<Ray2 *>(*it1)),
									*(static_cast<Ray2 *>(*it2)),
									control,w))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Box") && (typ2=="Box")) 
			{
			  if ( TestIntersection(*(static_cast<Box2 *>(*it1)),
									*(static_cast<Box2 *>(*it2))))
				return true;
			}
		  //-----------------------------------------------------------
		  //               Differents objets intersections
		  //-----------------------------------------------------------
		  //*******
		  // Line    / Circle
		  // Ray     / Circle
		  // Segment / Circle
		  //*******
		  //-----------------------------------------------------------
		  if ((typ1=="Line") && (typ2=="Circle"))
			{
			  if ( FindIntersection(*(static_cast<Line2 *>(*it1)),
									*(static_cast<Circle2 *>(*it2)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Circle") && (typ2=="Line"))
			{
			  if ( FindIntersection(*(static_cast<Line2 *>(*it2)),
									*(static_cast<Circle2 *>(*it1)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Ray") && (typ2=="Circle"))
			{
			  if ( FindIntersection(*(static_cast<Ray2 *>(*it1)),
									*(static_cast<Circle2 *>(*it2)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Circle") && (typ2=="Ray"))
			{
			  if ( FindIntersection(*(static_cast<Ray2 *>(*it2)),
									*(static_cast<Circle2 *>(*it1)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Segment") && (typ2=="Circle"))
			{
			  if ( FindIntersection(*(static_cast<Segment2 *>(*it1)),
									*(static_cast<Circle2 *>(*it2)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Circle") && (typ2=="Segment"))
			{
			  if ( FindIntersection(*(static_cast<Segment2 *>(*it2)),
									*(static_cast<Circle2 *>(*it1)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  //*******
		  // Line    / Arc
		  // Ray     / Arc
		  // Segment / Arc
		  //*******
		  //-----------------------------------------------------------
		  if ((typ1=="Line") && (typ2=="Arc"))
			{
			  if ( FindIntersection(*(static_cast<Line2 *>(*it1)),
									*(static_cast<Arc2 *>(*it2)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Arc") && (typ2=="Line"))
			{
			  if ( FindIntersection(*(static_cast<Line2 *>(*it2)),
									*(static_cast<Arc2 *>(*it1)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Ray") && (typ2=="Arc"))
			{
			  if ( FindIntersection(*(static_cast<Ray2 *>(*it1)),
									*(static_cast<Arc2 *>(*it2)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Arc") && (typ2=="Ray"))
			{
			  if ( FindIntersection(*(static_cast<Ray2 *>(*it2)),
									*(static_cast<Arc2 *>(*it1)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Segment") && (typ2=="Arc"))
			{
			  if ( FindIntersection(*(static_cast<Segment2 *>(*it1)),
									*(static_cast<Arc2 *>(*it2)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Arc") && (typ2=="Segment"))
			{
			  if ( FindIntersection(*(static_cast<Segment2 *>(*it2)),
									*(static_cast<Arc2 *>(*it1)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  //*******
		  // Line    / Ray
		  // Ray     / Segment
		  // Line    / Segment
		  //*******
		  //-----------------------------------------------------------
		  if ((typ1=="Line") && (typ2=="Ray")) 
			{
			  if ( FindIntersection(*(static_cast<Line2 *>(*it1)),
									*(static_cast<Ray2 *>(*it2)),
									control,w))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Ray") && (typ2=="Line")) 
			{
			  if ( FindIntersection(*(static_cast<Line2 *>(*it2)),
									*(static_cast<Ray2 *>(*it1)),
									control,w))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Line") && (typ2=="Segment")) 
			{
			  if ( FindIntersection(*(static_cast<Line2 *>(*it1)),
									*(static_cast<Segment2 *>(*it2)),
									control,w))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Segment") && (typ2=="Line")) 
			{
			  if ( FindIntersection(*(static_cast<Line2 *>(*it2)),
									*(static_cast<Segment2 *>(*it1)),
									control,w))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Ray") && (typ2=="Segment")) 
			{
			  if ( FindIntersection(*(static_cast<Ray2 *>(*it1)),
									*(static_cast<Segment2 *>(*it2)),
									control,w))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Segment") && (typ2=="Ray")) 
			{
			  if ( FindIntersection(*(static_cast<Ray2 *>(*it2)),
									*(static_cast<Segment2 *>(*it1)),
									control,w))
				return true;
			}
		  //-----------------------------------------------------------
		  //*******
		  // Circle    / Arc
		  //*******
		  //-----------------------------------------------------------
		  if ((typ1=="Circle") && (typ2=="Arc"))
			{
			  if ( FindIntersection(*(static_cast<Circle2 *>(*it1)),
									*(static_cast<Arc2 *>(*it2)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Arc") && (typ2=="Circle"))
			{
			  if ( FindIntersection(*(static_cast<Circle2 *>(*it2)),
									*(static_cast<Arc2 *>(*it1)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  //Box / Circle
		  //Box / Arc WARNING Incorrect detection Arc is treated as Circle !!!!
		  //Box / line
		  //Box / Ray
		  //Box / Segment
		  //
		  
		  if ((typ1=="Box")&&((typ2=="Circle")||(typ2=="Arc")))
			{
			  if ( TestIntersection(*(static_cast<Box2 *>(*it1)),
									*(static_cast<Circle2 *>(*it2))))
				return true;
			}
		  if (((typ1=="Circle")||(typ2=="Arc"))&&(typ2=="Box"))
			{
			  if ( TestIntersection(*(static_cast<Circle2 *>(*it1)),
									*(static_cast<Box2 *>(*it2))))
				return true;
			}
		  
		  if ((typ1=="Box")&&(typ2=="Line"))
			{
			  if ( TestIntersection(*(static_cast<Box2 *>(*it1)),
									*(static_cast<Line2 *>(*it2))))
				return true;
			}
		  
		  if ((typ1=="Box")&&(typ2=="Ray"))
			{
			  if ( TestIntersection(*(static_cast<Box2 *>(*it1)),
									*(static_cast<Ray2 *>(*it2))))
				return true;
			}
		  
		  if ((typ1=="Box")&&(typ2=="Segment"))
			{
			    if ( TestIntersection(*(static_cast<Box2 *>(*it1)),
			  						*(static_cast<Segment2 *>(*it2))))
				return true;
			}
		  if ((typ1=="Line")&&(typ2=="Box"))
			{
			  if ( TestIntersection(*(static_cast<Box2 *>(*it2)),
									*(static_cast<Line2 *>(*it1))))
				return true;
			}
		  
		  if ((typ1=="Ray")&&(typ2=="Box"))
			{
			  if ( TestIntersection(*(static_cast<Box2 *>(*it2)),
									*(static_cast<Ray2 *>(*it1))))
				return true;
			}
		  
		  if ((typ1=="Segment")&&(typ2=="Box"))
			{
			  if ( TestIntersection(*(static_cast<Box2 *>(*it2)),
			  						*(static_cast<Segment2 *>(*it1))))
				return true;
			}
		  

#else
		  //-----------------------------------------------------------
		  //               Same objets intersections
		  //-----------------------------------------------------------
		  if ((typ1=="Circle") && (typ2=="Circle")) 
			{
			  if ( FindIntersection(*(dynamic_cast<Circle2 *>(*it1)),
									*(dynamic_cast<Circle2 *>(*it2)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Arc") && (typ2=="Arc")) 
			{
			  if ( FindIntersection(*(dynamic_cast<Arc2 *>(*it1)),
									*(dynamic_cast<Arc2 *>(*it2)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Segment") && (typ2=="Segment")) 
			{
			  if ( FindIntersection(*(dynamic_cast<Segment2 *>(*it1)),
									*(dynamic_cast<Segment2 *>(*it2)),
									control,w))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Line") && (typ2=="Line")) 
			{
			  if ( FindIntersection(*(dynamic_cast<Line2 *>(*it1)),
									*(dynamic_cast<Line2 *>(*it2)),
									control,w))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Ray") && (typ2=="Ray")) 
			{
			  if ( FindIntersection(*(dynamic_cast<Ray2 *>(*it1)),
									*(dynamic_cast<Ray2 *>(*it2)),
									control,w))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Box") && (typ2=="Box")) 
			{
			  if ( TestIntersection(*(dynamic_cast<Box2 *>(*it1)),
									*(dynamic_cast<Box2 *>(*it2))))
				return true;
			}
		  //-----------------------------------------------------------
		  //               Differents objets intersections
		  //-----------------------------------------------------------
		  //*******
		  // Line    / Circle
		  // Ray     / Circle
		  // Segment / Circle
		  //*******
		  //-----------------------------------------------------------
		  if ((typ1=="Line") && (typ2=="Circle"))
			{
			  if ( FindIntersection(*(dynamic_cast<Line2 *>(*it1)),
									*(dynamic_cast<Circle2 *>(*it2)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Circle") && (typ2=="Line"))
			{
			  if ( FindIntersection(*(dynamic_cast<Line2 *>(*it2)),
									*(dynamic_cast<Circle2 *>(*it1)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Ray") && (typ2=="Circle"))
			{
			  if ( FindIntersection(*(dynamic_cast<Ray2 *>(*it1)),
									*(dynamic_cast<Circle2 *>(*it2)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Circle") && (typ2=="Ray"))
			{
			  if ( FindIntersection(*(dynamic_cast<Ray2 *>(*it2)),
									*(dynamic_cast<Circle2 *>(*it1)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Segment") && (typ2=="Circle"))
			{
			  if ( FindIntersection(*(dynamic_cast<Segment2 *>(*it1)),
									*(dynamic_cast<Circle2 *>(*it2)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Circle") && (typ2=="Segment"))
			{
			  if ( FindIntersection(*(dynamic_cast<Segment2 *>(*it2)),
									*(dynamic_cast<Circle2 *>(*it1)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  //*******
		  // Line    / Arc
		  // Ray     / Arc
		  // Segment / Arc
		  //*******
		  //-----------------------------------------------------------
		  if ((typ1=="Line") && (typ2=="Arc"))
			{
			  if ( FindIntersection(*(dynamic_cast<Line2 *>(*it1)),
									*(dynamic_cast<Arc2 *>(*it2)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Arc") && (typ2=="Line"))
			{
			  if ( FindIntersection(*(dynamic_cast<Line2 *>(*it2)),
									*(dynamic_cast<Arc2 *>(*it1)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Ray") && (typ2=="Arc"))
			{
			  if ( FindIntersection(*(dynamic_cast<Ray2 *>(*it1)),
									*(dynamic_cast<Arc2 *>(*it2)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Arc") && (typ2=="Ray"))
			{
			  if ( FindIntersection(*(dynamic_cast<Ray2 *>(*it2)),
									*(dynamic_cast<Arc2 *>(*it1)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Segment") && (typ2=="Arc"))
			{
			  if ( FindIntersection(*(dynamic_cast<Segment2 *>(*it1)),
									*(dynamic_cast<Arc2 *>(*it2)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Arc") && (typ2=="Segment"))
			{
			  if ( FindIntersection(*(dynamic_cast<Segment2 *>(*it2)),
									*(dynamic_cast<Arc2 *>(*it1)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  //*******
		  // Line    / Ray
		  // Ray     / Segment
		  // Line    / Segment
		  //*******
		  //-----------------------------------------------------------
		  if ((typ1=="Line") && (typ2=="Ray")) 
			{
			  if ( FindIntersection(*(dynamic_cast<Line2 *>(*it1)),
									*(dynamic_cast<Ray2 *>(*it2)),
									control,w))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Ray") && (typ2=="Line")) 
			{
			  if ( FindIntersection(*(dynamic_cast<Line2 *>(*it2)),
									*(dynamic_cast<Ray2 *>(*it1)),
									control,w))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Line") && (typ2=="Segment")) 
			{
			  if ( FindIntersection(*(dynamic_cast<Line2 *>(*it1)),
									*(dynamic_cast<Segment2 *>(*it2)),
									control,w))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Segment") && (typ2=="Line")) 
			{
			  if ( FindIntersection(*(dynamic_cast<Line2 *>(*it2)),
									*(dynamic_cast<Segment2 *>(*it1)),
									control,w))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Ray") && (typ2=="Segment")) 
			{
			  if ( FindIntersection(*(dynamic_cast<Ray2 *>(*it1)),
									*(dynamic_cast<Segment2 *>(*it2)),
									control,w))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Segment") && (typ2=="Ray")) 
			{
			  if ( FindIntersection(*(dynamic_cast<Ray2 *>(*it2)),
									*(dynamic_cast<Segment2 *>(*it1)),
									control,w))
				return true;
			}
		  //-----------------------------------------------------------
		  //*******
		  // Circle    / Arc
		  //*******
		  //-----------------------------------------------------------
		  if ((typ1=="Circle") && (typ2=="Arc"))
			{
			  if ( FindIntersection(*(dynamic_cast<Circle2 *>(*it1)),
									*(dynamic_cast<Arc2 *>(*it2)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  if ((typ1=="Arc") && (typ2=="Circle"))
			{
			  if ( FindIntersection(*(dynamic_cast<Circle2 *>(*it2)),
									*(dynamic_cast<Arc2 *>(*it1)),
									control,wcir))
				return true;
			}
		  //-----------------------------------------------------------
		  //Box / Circle
		  //Box / Arc WARNING Incorrect detection Arc is treated as Circle !!!!
		  //Box / line
		  //Box / Ray
		  //Box / Segment
		  //
		  
		  if ((typ1=="Box")&&((typ2=="Circle")||(typ2=="Arc")))
			{
			  if ( TestIntersection(*(dynamic_cast<Box2 *>(*it1)),
									*(dynamic_cast<Circle2 *>(*it2))))
				return true;
			}
		  if (((typ1=="Circle")||(typ2=="Arc"))&&(typ2=="Box"))
			{
			  if ( TestIntersection(*(dynamic_cast<Circle2 *>(*it1)),
									*(dynamic_cast<Box2 *>(*it2))))
				return true;
			}
		  
		  if ((typ1=="Box")&&(typ2=="Line"))
			{
			  if ( TestIntersection(*(dynamic_cast<Box2 *>(*it1)),
			  					*(dynamic_cast<Line2 *>(*it2))))
				return true;
			}
		  
		  if ((typ1=="Box")&&(typ2=="Ray"))
			{
			  if ( TestIntersection(*(dynamic_cast<Box2 *>(*it1)),
									*(dynamic_cast<Ray2 *>(*it2))))
				return true;
			}
		  
		  if ((typ1=="Box")&&(typ2=="Segment"))
			{
			  if ( TestIntersection(*(dynamic_cast<Box2 *>(*it1)),
			  					*(dynamic_cast<Segment2 *>(*it2))))
				return true;
			}
		  if ((typ1=="Line")&&(typ2=="Box"))
			{
			  if ( TestIntersection(*(dynamic_cast<Box2 *>(*it2)),
									*(dynamic_cast<Line2 *>(*it1))))
				return true;
			}
		  
		  if ((typ1=="Ray")&&(typ2=="Box"))
			{
			  if ( TestIntersection(*(dynamic_cast<Box2 *>(*it2)),
									*(dynamic_cast<Ray2 *>(*it1))))
				return true;
			}
		  
		  if ((typ1=="Segment")&&(typ2=="Box"))
			{
			  if ( TestIntersection(*(dynamic_cast<Box2 *>(*it2)),
			  					*(dynamic_cast<Segment2 *>(*it1))))
				return true;
			}
#endif
		  
		  
		  
		}	  
	}
  
  return false; // if no collision
}
//----------------------------------------------------------------------------

bool Geom::GeomFormCollisionDetection ( Form& _form1,  vector<Form*>& _vform2)
{
  for (vector<Form*>::iterator it1=_vform2.begin();it1!=_vform2.end();it1++) 
		  {
			if (GeomFormCollisionDetection(_form1,(**it1))) 
			  {
				return true;
			  }
		  }
  return false;
  
}

  

