
#include "GeomIntr2DCirBox.h"
using namespace Geom;

//----------------------------------------------------------------------------
// All functions return -1 if initially intersecting, 0 if no intersection,
// +1 if intersects at some positive time.
//----------------------------------------------------------------------------
static bool TestVertexRegion (Real fCx, Real fCy, Real fR, Real fEx, Real fEy)
{
    Real fDx = fCx + fEx;
    Real fDy = fCy + fEy;
    Real fRSqr = fR*fR;
    Real fDiff = fDx*fDx + fDy*fDy - fRSqr;
    if ( fDiff <= 0.0f )
    {
        // circle is already intersecting the box
	  return true;
    }

    return false;
}
//----------------------------------------------------------------------------
static bool TestEdgeRegion (Real fCx, Real fCy, Real fR, Real fEx, Real fEy)
{
    Real fDx = fCx + fEx;
    Real fXSignedDist = fDx + fR;
    if ( fXSignedDist >= 0.0f )
    {
        // circle is already intersecting the box
	  return true;
    }
    return false;
}
//----------------------------------------------------------------------------
bool Geom::TestIntersection (const Box2& _Box,const Circle2& _Circle)
{
    // Convert circle center to box coordinates.
    Vector2 kDiff = _Circle.Center() - _Box.Center();
    Real fCx = kDiff.Dot(_Box.Axis(0));
    Real fCy = kDiff.Dot(_Box.Axis(1));
    Real fEx = _Box.Extent(0);
    Real fEy = _Box.Extent(1);
	Real fR  = _Circle.Radius();


    bool iType = false;

    if ( fCx < -fEx )
    {
        if ( fCy < -fEy )
        {
            // region Rmm
            iType = TestVertexRegion(fCx,fCy,fR,fEx,fEy);
        }
        else if ( fCy <= fEy )
        {
            // region Rmz
            iType = TestEdgeRegion(fCx,fCy,fR,fEx,fEy);
        }
        else
        {
            // region Rmp
            iType = TestVertexRegion(fCx,-fCy,fR,fEx,fEy);
        }
    }
    else if ( fCx <= fEx )
    {
        if ( fCy < -fEy )
        {
            // region Rzm
            iType = TestEdgeRegion(fCy,fCx,fR,fEy,fEx);
        }
        else if ( fCy <= fEy )
        {
            // region Rzz: circle is already intersecting the box
            return true;
        }
        else
        {
            // region Rzp
            iType = TestEdgeRegion(-fCy,fCx,fR,fEy,fEx);
        }
    }
    else
    {
        if ( fCy < -fEy )
        {
            // region Rpm
            iType = TestVertexRegion(-fCx,fCy,fR,fEx,fEy);
        }
        else if ( fCy <= fEy )
        {
            // region Rpz
            iType = TestEdgeRegion(-fCx,fCy,fR,fEx,fEy);
        }
        else
        {
            // region Rpp
            iType = TestVertexRegion(-fCx,-fCy,fR,fEx,fEy);
        }
    }

	return iType;
}
//----------------------------------------------------------------------------
bool Geom::TestIntersection (const Circle2& _Circle,const Box2& _Box)
{
  return (TestIntersection(_Box,_Circle));
}
