#ifndef GEOMLINE2_H
#define GEOMLINE2_H

// Two different representations of the line are allowed.  If one form is
// used, the other is *not* automatically set.

#include "GeomPrimitivForm.h"
#include "MagicFMLibType.h"
#include "GeomVector2.h"
#include <string>
using namespace std;


namespace Geom {


  class MAGICFM Line2 : public PrimitivForm
{
public:
    Line2 ();

	// parametric form:
    //   origin = V0, direction = V1-V0
    //
    // normal form:
    //   normal = Perp(V1-V0) where Perp(x,y)=(y,-x),
    //   constant = Dot(normal,V0)
    Line2 (const Vector2& _V0, const Vector2& _V1, bool bParametricForm);

	string type() {return "Line";}
	
    // Line is L(t) = P+t*D for any real-valued t.  D is not necessarily
    // unit length.
    Vector2& Origin ();
    const Vector2& Origin () const;
    Vector2& Direction ();
    const Vector2& Direction () const;

    // Line is Dot(N,X) = c.  N is not necessarily unit length.
    Vector2& Normal ();
    const Vector2& Normal () const;
    Real& Constant ();
    const Real& Constant () const;

    // Compute d = Dot(N,V)-c.  The value |d| is the true distance from V
    // to the line whenever |N| = 1.
    Real GetPseudodistance (const Vector2& _V) const;

	// rotations & translations
	virtual void Translate(Real _tx, Real _ty)
	  {
		m_kOrigin.Translate(_tx,_ty);
	  }
	virtual void Rotate(Real _cost, Real _sint, Real _centerx, Real _centery)
	  {
		m_kOrigin.Rotate(_cost,_sint,_centerx,_centery);
		m_kDirection.Rotate(_cost,_sint,_centerx,_centery);
		
	  }
	virtual void Rotate(Real _theta, Real _centerx, Real _centery)
	  {
		m_kOrigin.Rotate(_theta,_centerx,_centery);
		m_kDirection.Rotate(_theta,_centerx,_centery);
	  }

protected:
    // parametric form
    Vector2 m_kOrigin;
    Vector2 m_kDirection;

    // normal form
    Vector2 m_kNormal;
    Real m_fConstant;
};

#include "GeomLine2.inl"

} // namespace Geom

#endif


