#include "GeomVector2.h"

using namespace Geom;

const Vector2 Vector2::ZERO(0.0f,0.0f);
const Vector2 Vector2::UNIT_X(1.0f,0.0f);
const Vector2 Vector2::UNIT_Y(0.0f,1.0f);
Real Vector2::FUZZ = 0.0f;

//----------------------------------------------------------------------------
Vector2::Vector2 (Real fX, Real fY)
{
    x = fX;
    y = fY;
}
//----------------------------------------------------------------------------
Vector2::Vector2 (Real afCoordinate[2])
{
    x = afCoordinate[0];
    y = afCoordinate[1];
}
//----------------------------------------------------------------------------
Vector2::Vector2 (const Vector2& _Vector)
{
    x = _Vector.x;
    y = _Vector.y;
}
//----------------------------------------------------------------------------
void Vector2::Translate(Real _tx, Real _ty)
{
  x+=_tx;
  y+=_ty;
}
//----------------------------------------------------------------------------
void Vector2::Rotate(Real _cost, Real _sint, Real _centerx, Real _centery)
{
  Real cx = (x-_centerx);
  Real cy = (y-_centery);
  x=(cx*_cost-cy*_sint)+_centerx;
  y=(cx*_sint+cy*_cost)+_centery;
  
}
//----------------------------------------------------------------------------
void Vector2::Rotate(Real _theta, Real _centerx, Real _centery)
{
  Rotate(cos(_theta),sin(_theta),_centerx,_centery);
}

//----------------------------------------------------------------------------
Vector2& Vector2::operator= (const Vector2& _Vector)
{
    x = _Vector.x;
    y = _Vector.y;
    return *this;
}
//----------------------------------------------------------------------------
bool Vector2::operator== (const Vector2& _Vector) const
{
    if ( FUZZ == 0.0f )
        return x == _Vector.x && y == _Vector.y;
    else
        return Math::FAbs(x-_Vector.x) <= FUZZ 
            && Math::FAbs(y-_Vector.y) <= FUZZ;
}
//----------------------------------------------------------------------------
bool Vector2::operator!= (const Vector2& _Vector) const
{
    if ( FUZZ == 0.0f )
        return x != _Vector.x || y != _Vector.y;
    else
        return Math::FAbs(x-_Vector.x) > FUZZ
             || Math::FAbs(y-_Vector.y) > FUZZ;
}
//----------------------------------------------------------------------------
bool Vector2::operator< (const Vector2& _Vector) const
{
    Real fXTmp = _Vector.x, fYTmp = _Vector.y;
    if ( FUZZ > 0.0f )
    {
        if ( Math::FAbs(x - fXTmp) <= FUZZ )
            fXTmp = x;
        if ( Math::FAbs(y - fYTmp) <= FUZZ )
            fYTmp = y;
    }

    // compare y values
    unsigned int uiTest0 = *(unsigned int*)&y;
    unsigned int uiTest1 = *(unsigned int*)&fYTmp;
    if ( uiTest0 < uiTest1 )
        return true;
    if ( uiTest0 > uiTest1 )
        return false;

    // compare x values
    uiTest0 = *(unsigned int*)&x;
    uiTest1 = *(unsigned int*)&fXTmp;
    return uiTest0 < uiTest1;
}
//----------------------------------------------------------------------------
bool Vector2::operator<= (const Vector2& _Vector) const
{
    Real fXTmp = _Vector.x, fYTmp = _Vector.y;
    if ( FUZZ > 0.0f )
    {
        if ( Math::FAbs(x - fXTmp) <= FUZZ )
            fXTmp = x;
        if ( Math::FAbs(y - fYTmp) <= FUZZ )
            fYTmp = y;
    }

    // compare y values
    unsigned int uiTest0 = *(unsigned int*)&y;
    unsigned int uiTest1 = *(unsigned int*)&fYTmp;
    if ( uiTest0 < uiTest1 )
        return true;
    if ( uiTest0 > uiTest1 )
        return false;

    // compare x values
    uiTest0 = *(unsigned int*)&x;
    uiTest1 = *(unsigned int*)&fXTmp;
    return uiTest0 <= uiTest1;
}
//----------------------------------------------------------------------------
bool Vector2::operator> (const Vector2& _Vector) const
{
    Real fXTmp = _Vector.x, fYTmp = _Vector.y;
    if ( FUZZ > 0.0f )
    {
        if ( Math::FAbs(x - fXTmp) <= FUZZ )
            fXTmp = x;
        if ( Math::FAbs(y - fYTmp) <= FUZZ )
            fYTmp = y;
    }

    // compare y values
    unsigned int uiTest0 = *(unsigned int*)&y;
    unsigned int uiTest1 = *(unsigned int*)&fYTmp;
    if ( uiTest0 > uiTest1 )
        return true;
    if ( uiTest0 < uiTest1 )
        return false;

    // compare x values
    uiTest0 = *(unsigned int*)&x;
    uiTest1 = *(unsigned int*)&fXTmp;
    return uiTest0 > uiTest1;
}
//----------------------------------------------------------------------------
bool Vector2::operator>= (const Vector2& _Vector) const
{
    Real fXTmp = _Vector.x, fYTmp = _Vector.y;
    if ( FUZZ > 0.0f )
    {
        if ( Math::FAbs(x - fXTmp) <= FUZZ )
            fXTmp = x;
        if ( Math::FAbs(y - fYTmp) <= FUZZ )
            fYTmp = y;
    }

    // compare y values
    unsigned int uiTest0 = *(unsigned int*)&y;
    unsigned int uiTest1 = *(unsigned int*)&fYTmp;
    if ( uiTest0 > uiTest1 )
        return true;
    if ( uiTest0 < uiTest1 )
        return false;

    // compare x values
    uiTest0 = *(unsigned int*)&x;
    uiTest1 = *(unsigned int*)&fXTmp;
    return uiTest0 >= uiTest1;
}
//----------------------------------------------------------------------------
Vector2 Vector2::operator+ (const Vector2& _Vector) const
{
    return Vector2(x+_Vector.x,y+_Vector.y);
}
//----------------------------------------------------------------------------
Vector2 Vector2::operator- (const Vector2& _Vector) const
{
    return Vector2(x-_Vector.x,y-_Vector.y);
}
//----------------------------------------------------------------------------
Vector2 Vector2::operator* (Real fScalar) const
{
    return Vector2(fScalar*x,fScalar*y);
}
//----------------------------------------------------------------------------
Vector2 Vector2::operator- () const
{
    return Vector2(-x,-y);
}
//----------------------------------------------------------------------------
Vector2 Geom::operator* (Real fScalar, const Vector2& _Vector)
{
    return Vector2(fScalar*_Vector.x,fScalar*_Vector.y);
}
//----------------------------------------------------------------------------
Vector2 Vector2::operator/ (Real fScalar) const
{
    Vector2 kQuot;

    if ( fScalar != 0.0f )
    {
        Real fInvScalar = 1.0f/fScalar;
        kQuot.x = fInvScalar*x;
        kQuot.y = fInvScalar*y;
        return kQuot;
    }
    else
    {
        return Vector2(Math::MAX_REAL,Math::MAX_REAL);
    }
}
//----------------------------------------------------------------------------
Vector2& Vector2::operator+= (const Vector2& _Vector)
{
    x += _Vector.x;
    y += _Vector.y;
    return *this;
}
//----------------------------------------------------------------------------
Vector2& Vector2::operator-= (const Vector2& _Vector)
{
    x -= _Vector.x;
    y -= _Vector.y;
    return *this;
}
//----------------------------------------------------------------------------
Vector2& Vector2::operator*= (Real fScalar)
{
    x *= fScalar;
    y *= fScalar;
    return *this;
}
//----------------------------------------------------------------------------
Vector2& Vector2::operator/= (Real fScalar)
{
    if ( fScalar != 0.0f )
    {
        Real fInvScalar = 1.0f/fScalar;
        x *= fInvScalar;
        y *= fInvScalar;
    }
    else
    {
        x = Math::MAX_REAL;
        y = Math::MAX_REAL;
    }

    return *this;
}
//----------------------------------------------------------------------------
Real Vector2::Dot (const Vector2& _Vector) const
{
    return x*_Vector.x + y*_Vector.y;
}
//----------------------------------------------------------------------------
Real Vector2::Length () const
{
    return Math::Sqrt(x*x +y*y);
}
//----------------------------------------------------------------------------
Vector2 Vector2::Cross () const
{
    return Vector2(y,-x);
}
//----------------------------------------------------------------------------
Vector2 Vector2::UnitCross () const
{
    Vector2 kCross(y,-x);
    kCross.Unitize();
    return kCross;
}
//----------------------------------------------------------------------------
Real Vector2::Unitize (Real fTolerance)
{
    Real fLength = Length();

    if ( fLength > fTolerance )
    {
        Real fInvLength = 1.0f/fLength;
        x *= fInvLength;
        y *= fInvLength;
    }
    else
    {
        fLength = 0.0f;
    }

    return fLength;
}
//----------------------------------------------------------------------------
void Vector2::Orthonormalize (Vector2 akVector[/*2*/])
{
    // If the input vectors are v0 and v1, then the Gram-Schmidt
    // orthonormalization produces vectors u0 and u1 as follows,
    //
    //   u0 = v0/|v0|
    //   u1 = (v1-(u0*v1)u0)/|v1-(u0*v1)u0|
    //
    // where |A| indicates length of vector A and A*B indicates dot
    // product of vectors A and B.

    // compute u0
    akVector[0].Unitize();

    // compute u1
    Real fDot0 = akVector[0].Dot(akVector[1]); 
    akVector[1] -= fDot0*akVector[0];
    akVector[1].Unitize();
}
//----------------------------------------------------------------------------


