#ifndef ENVIRONMENT_H
#define ENVIRONMENT_H

//#include "global.h"
//#include "math.h"
//#include "misc.h"

#include <vector> 
#include <string>
#include "global.h"
#include "exception.h"
#include "../GEOMsources/GeomFormCollisionDetection.h"
#include "light.h"
#include "GLheader.h"
//#include "robot.h"

#include <iostream>
#include <stdio.h>
#include <fstream>

//extern void altern_position(robot * robot_provi);
extern int winIdMain;
using namespace std; // for string and gcc3.2
using namespace Geom;


class obstacle  //Abstract class, all the obstacles must derive from this class

{
 public: 
    bool can_be_gripped_;
	bool gripped;
  
    string name;
	Form* pforme;
	GLint id_obstacle; // to save the GLstructure of the walls/cylinders
	
    //  virtual bool collision (double & _x, double & _y) = 0; //To detect a collision between a robot and an obstable

	virtual string TypeObstacle() =0;  //To return the type of the obstacle, it is useful so that the robot can update this sensors values
	virtual void initGL3D() =0;
	virtual void GL3D_without_Lists()=0;
	virtual void display_2D()=0;
	virtual void display_3D()=0;
	virtual void save(FILE *Fichier)=0;
	virtual void can_be_gripped()=0;
	virtual void grip()=0;
	virtual void release()=0;

	void Translate (Real _tx,Real _ty)
	  {
		pforme->Translate(_tx,_ty);
	  }
	

	void Rotate (Real _theta,Real _centerx, Real _centery)
	  {
		pforme->Rotate(_theta,_centerx,_centery);
		
	  }
	
	
	
	Form* give_form()
	  {
		if (!gripped)
		  {
			return pforme;
		  }
		else
		  {
			return EMPTY_FORM_POINTER;
		  }
	  }
	

	obstacle(string  _name,bool _bgr=false) 

	{
	    name=_name;

		gripped=false;
		can_be_gripped_=_bgr;
		
	}

	virtual ~obstacle()
	  {

	  }
	
	
	/*
	void display_3D()
	{
	  glPushMatrix();
	  
	  if (glutGetWindow()==winIdMain)
		{
		  glCallList(id_obstacle); 
		} else {
		  GL3D_without_Lists();
		}
  	  glPopMatrix();
	}
	*/
	
	//  double x11();//{return 0.0;}

};


class wall : public obstacle //  [(x1,y1);(x2,y2)] is the wall segment

{

 public:

    wall(string _name,double  _x1,double  _y1,double  _x2,double  _y2,unsigned  _color=255,bool _bgr=false) : obstacle(_name,_bgr) 
	{
	  //  color=_color;

		Segment2 *seg1= new Segment2;
		seg1->Origin()   =Vector2(_x1,_y1);
		seg1->Direction()=Vector2(_x2-_x1,_y2-_y1);

		vector<PrimitivForm*> *pf =new vector <PrimitivForm *>;
		pf->push_back(seg1);
		pforme =new Form(*pf,_color);

		// GL initialisations
		initGL3D();
	}

    virtual string TypeObstacle() {return "WALL";}

	virtual void initGL3D();
	  
	virtual void GL3D_without_Lists();

	virtual void can_be_gripped();
	
	virtual void grip();

	virtual void release();
	
	void display_2D();
	  
	void display_3D();
	
	
	
	virtual ~wall()
	  {
		for (vector<PrimitivForm*>::iterator it1=(pforme->forms).begin();it1!=(pforme->forms).end();it1++) 
		  {
			delete (*it1);
		  }
		delete pforme;
		glDeleteLists(id_obstacle,1);
	  };

	
	virtual void save(FILE *Fichier);

};

class cylinder : public obstacle //  (x,y) is the center of the cylinder, and r the radius

{
 protected:  

    double distcollision;

 public:
	/*    double  x;
    double  y;
    double  r;*/
	//    unsigned  color;

    cylinder(string _name, double  _x,double  _y, double  _r,unsigned  _color=255,bool _bgr=false)  : obstacle (_name,_bgr)  
	{
	  /*x=_x;
	    y=_y;
	    r=_r;
	    color=_color;*/
		
		Circle2 *cir1= new Circle2;
		cir1->Center()=Vector2(_x,_y);
		cir1->Radius()=_r;
		vector<PrimitivForm*> *pf =new vector <PrimitivForm *>;
		pf->push_back(cir1);
		pforme =new Form(*pf,_color);

		// GL initialisations
		initGL3D();
	
	}

    virtual string TypeObstacle() {return "CYLINDER";}

	virtual void initGL3D();
	virtual void GL3D_without_Lists();
	virtual void display_2D();
	virtual void display_3D();
	virtual void save(FILE *Fichier);
	virtual void can_be_gripped();
	virtual void grip();
	virtual void release();

	
	
	virtual ~cylinder()
	  {
		for (vector<PrimitivForm*>::iterator it1=(pforme->forms).begin();it1!=(pforme->forms).end();it1++) 
		  {
			delete (*it1);
		  }
		delete pforme;
		glDeleteLists(id_obstacle,1);
	  };

	
	
};

class cylinder_unfixed : public cylinder //  (x,y) is the center of the cylinder, and r the radius

{
 public:
    cylinder_unfixed (string _name,double  _x,double  _y, double  _r,unsigned  _color,bool _bgr=false) : cylinder(_name,_x, _y, _r, _color,_bgr) {
	  
		// GL initialisations
		initGL3D();
	}

    virtual string TypeObstacle() {return "CYLINDER_UNFIXED";}

	//    virtual ~cylinder_unfixed(){};
	virtual void initGL3D();
	
	virtual void GL3D_without_Lists();
	
	virtual void display_2D();
	
	virtual void display_3D();
	
	virtual void save(FILE *Fichier);
	
	virtual void can_be_gripped();
	
	virtual void grip();

	virtual void release();

	
};

class bloc : public obstacle //  (x1,y1) is the center of the bloc
// (x2,y2) the vector for axe 1
// (x3,y3) the vector for axe 2


{

 public:

    bloc(string _name,double  _x1,double  _y1,double  _x2,double  _y2,double _x3,double _y3,unsigned  _color=255,bool _bgr=false) : obstacle(_name,_bgr) 
	{
		Box2 *bloc1= new Box2;
		
		bloc1->Center()   =Vector2(_x1,_y1);

		Real len1=(Vector2(_x2,_y2)).Length();
		Real len2=(Vector2(_x3,_y3)).Length();
		
		bloc1->Axes()[0]  =(Vector2(_x2,_y2))/len1;
		bloc1->Axes()[1]  =(Vector2(_x3,_y3))/len2;
		bloc1->Extents()[0]=len1;
		bloc1->Extents()[1]=len2;
		
		vector<PrimitivForm*> *pf =new vector <PrimitivForm *>;
		pf->push_back(bloc1);
		pforme =new Form(*pf,_color);

		// GL initialisations
		initGL3D();
		
	}

    virtual string TypeObstacle() {return "BLOC";}

	virtual void initGL3D();
	
	virtual void GL3D_without_Lists();
	
	void display_2D();
	
	void display_3D();
	
	virtual void save(FILE *Fichier);

	virtual void can_be_gripped();
	
	virtual void grip();

	virtual void release();


	virtual ~bloc()
	  {
		for (vector<PrimitivForm*>::iterator it1=(pforme->forms).begin();it1!=(pforme->forms).end();it1++) 
		  {
			delete (*it1);
		  }
		delete pforme;
		glDeleteLists(id_obstacle,1);
	  };

};


class Food : public obstacle
{
  public :
	virtual string TypeObstacle() {return "FOOD";}
  
  virtual void initGL3D();
  virtual void GL3D_without_Lists();
	
  Food(Real & _x,Real & _y, Real _size,bool _bgr=false):obstacle("Food",_bgr)
	  {
		Circle2 *cir1= new Circle2;
		cir1->Center()=Vector2(_x,_y);
		cir1->Radius()=_size;
		vector<PrimitivForm*> *pf =new vector <PrimitivForm *>;
		pf->push_back(cir1);
		pforme =new Form(*pf,unsigned(255));

		// GL initialisations
		initGL3D();
	
	  }

  virtual void display_2D();
  virtual void display_3D();
  virtual void save(FILE *Fichier);
  virtual void can_be_gripped();
  virtual void grip();
  virtual void release();

		 
	
  virtual ~Food()
	{
	  for (vector<PrimitivForm*>::iterator it1=(pforme->forms).begin();it1!=(pforme->forms).end();it1++) 
		{
		  delete (*it1);
		}
	  delete pforme;
	  glDeleteLists(id_obstacle,1);
	};	
  
  
};


class environment

{
 protected:
  GLint id_env;
  
    public :

	  double    dx, dy; // world charthesian extension                  
	float  gl_shift_cam_x;	// used for having the envrt on the center of the 3D scene
	float  gl_shift_cam_y;	
		
    vector<obstacle *>  all_obstacles;
	vector<obstacle *>  all_obstacles_unfixed;
	vector<obstacle *>  all_foods;
    vector<light *>  all_lights;
	
    bool rebondlights;
  
    bool name_of_obstacle_is_already_used ( string _name)
	{if (all_obstacles.size()==0) return false;
	unsigned i;
	for (i=0;i<(all_obstacles.size());i++)
	    {
		if ((all_obstacles[i]->name)==_name) return true;
		 
	    }
	for (i=0;i<(all_obstacles_unfixed.size());i++)
	    {
		if ((all_obstacles_unfixed[i]->name)==_name) return true;
		 
	    }
	
	return false;

	}

    bool name_of_light_is_already_used ( string _name)
	{if (all_lights.size()==0) return false;
	for (unsigned i=0;i<(all_lights.size());i++)
	    {
		if ((all_lights[i]->name)==_name) return true;
	    }

	return false;

	}

    void add_wall (string _name, double  _x1,double  _y1,double  _x2,double  _y2,unsigned  _color=255,bool _b=false)
	{
	    if (name_of_obstacle_is_already_used (_name)) throw breakerror ("You can't use the same name for different obstacles");
	    all_obstacles.push_back(new wall(_name,_x1,_y1,_x2,_y2,_color,_b));
	}

    void add_cylinder (string _name, double  _x,double  _y, double  _r,unsigned  _color=255,bool _b=false)
	{
	    if (name_of_obstacle_is_already_used (_name)) throw breakerror ("You can't use the same name for different obstacles");
	    all_obstacles.push_back(new cylinder (_name,_x,_y,_r,_color,_b));
	}

    void add_cylinder_unfixed (string _name, double  _x,double  _y, double  _r,unsigned  _color=255,bool _b=false)
	{
	    if (name_of_obstacle_is_already_used (_name)) throw breakerror ("You can't use the same name for different obstacles");
	    all_obstacles_unfixed.push_back(new cylinder_unfixed (_name,_x,_y,_r,_color,_b));
	}
	void add_bloc (string _name, double  _x1,double  _y1,double  _x2,double  _y2,double  _x3,double  _y3,unsigned  _color=255,bool _b=false)
	{
	    if (name_of_obstacle_is_already_used (_name)) throw breakerror ("You can't use the same name for different obstacles");
	    all_obstacles.push_back(new bloc (_name,_x1,_y1,_x2,_y2,_x3,_y3,_color,_b));
	}
	void add_food(Real _x,Real _y,Real _size = 10.0,bool _b=false)
	  {
		all_foods.push_back(new Food(_x,_y,_size,_b));
	  }
	

    void add_light (string _name, double _x,double _y, double _power)
	{
	    if (name_of_light_is_already_used (_name)) throw breakerror ("You can't use the same name for different lights");
	    all_lights.push_back(new light (_name,_x, _y,  _power));
	}


	void initGL3D();
	void GL3D_without_Lists();
	void display_2D();
	void display_3D();
	void remove_all();
	void save(const char* _name);
	void load(const char* _name);
	
	  
	
	environment(double _dx, double _dy) 
	{
	    dx=_dx;
	    dy=_dy;
	    rebondlights=false;

		// used for having the environnement at the center of the 3D representation
		gl_shift_cam_x = -(dx)/(2*SCALE_GL);
		gl_shift_cam_y = -(dy)/(2*SCALE_GL);
		
	    //vector<light*> * all_lightstemp =NULL;
	    //all_lights=all_lightstemp;
		
		//GL initialisations
		initGL3D();
	}


	void change_dimension(double _dx, double _dy);
	
	static int maptoswitch(const char * buf)
	  {
		if (strcmp(buf,"wall")==0) {return 0;}
		if (strcmp(buf,"cylinder")==0) {return 1;}
		if (strcmp(buf,"cylinder_unfixed")==0) {return 2;}
		if (strcmp(buf,"bloc")==0) {return 3;}
		if (strcmp(buf,"food")==0) {return 4;}
		if (strcmp(buf,"light")==0){return 5;}
		
		return -1;
		
	  }
	
	
	//Constructor with a saved file
	environment(const char*  _name) 
	{
	  id_env=glGenLists(1); // To prepare GL initialisations
	  load(_name);
	  
	}
	
    ~environment()
	{
	  unsigned i;
	  
	  for (i=0;i<all_obstacles.size();i++)
		{delete all_obstacles[i];
		}
	  
	  for (i=0;i<all_lights.size();i++)
		{ delete all_lights[i];
		}
	  
	  for (i=0;i<all_foods.size();i++)
		{delete all_foods[i];
		}
	  
	  glDeleteLists(id_env,1);
	}

};


#endif
