#include "gripper.h"


using namespace Geom;


bool Gripper::gripper_should_move()
{
  return angle_have_changed;
}

void Gripper::gripper_set_angle(int angle)
{
  if (angle!=gripper_angle)
	{
	  angle_have_changed=true;
	  gripper_angle=angle;
	}
  
}
 
void Gripper::move(int khepera_fd)
{
  if (use_real_khepera) {
	
	const unsigned TEXT_BUF_SIZE=255;
	char message[TEXT_BUF_SIZE];
	char result[TEXT_BUF_SIZE];
	if (gripper_should_move()) {
	  sprintf(message,"T,1,E,%i\n",gripper_angle);
	  khep_talk(khepera_fd,message,result,TEXT_BUF_SIZE-1);
	  angle_have_changed=false;
	}
	
  }
	
  
}

void Gripper::try_to_grip(vector<obstacle *> _obs,Real _robotx,Real _roboty,Real _direction)
{

  Real vx,vy,x1,x2,x3,x4,y1,y2,y3,y4;
  
  vx=Math::Cos(_direction);
  vy=Math::Sin(_direction);
  
  x1=_robotx+vx*MIN_LENGTH_OF_GRIPPER+(-vy)*WIDTH_OF_GRIPPER;
  y1=_roboty+vy*MIN_LENGTH_OF_GRIPPER+(vx)*WIDTH_OF_GRIPPER;
  x2=_robotx+vx*MIN_LENGTH_OF_GRIPPER-(-vy)*WIDTH_OF_GRIPPER;
  y2=_roboty+vy*MIN_LENGTH_OF_GRIPPER-(vx)*WIDTH_OF_GRIPPER;
  x3=_robotx+vx*MAX_LENGTH_OF_GRIPPER+(-vy)*WIDTH_OF_GRIPPER;
  y3=_roboty+vy*MAX_LENGTH_OF_GRIPPER+(vx)*WIDTH_OF_GRIPPER;
  x4=_robotx+vx*MAX_LENGTH_OF_GRIPPER-(-vy)*WIDTH_OF_GRIPPER;
  y4=_roboty+vy*MAX_LENGTH_OF_GRIPPER-(vx)*WIDTH_OF_GRIPPER;  

  Segment2 *seg1= new Segment2;
  seg1->Origin()   =Vector2(x1,y1);
  seg1->Direction()=Vector2(x4-x1,y4-y1);

  Segment2 *seg2= new Segment2;
  seg2->Origin()   =Vector2(x2,y2);
  seg2->Direction()=Vector2(x3-x2,y3-y2);

  Segment2 *seg3= new Segment2;
  seg3->Origin()   =Vector2((x1+x3)/2,(y1+y3)/2);
  seg3->Direction()=Vector2((x4+x2)/2-(x1+x3)/2,(y4+y2)/2-(y1+y3)/2);

  Segment2 *seg4= new Segment2;
  seg4->Origin()   =Vector2((x1+x2)/2,(y1+y2)/2);
  seg4->Direction()=Vector2((x3+x4)/2-(x1+x2)/2,(y3+y4)/2-(y1+y2)/2);

  
  vector<PrimitivForm*> *pf =new vector <PrimitivForm *>;
  pf->push_back(seg1);
  pf->push_back(seg2);
  pf->push_back(seg3);
  pf->push_back(seg4);

  /*
	Form of the gripping area
	\ | /
     \|/
	 ---
	 /|\
	/ | \
	 
   */
  
  Form* gripper_area;
	
  gripper_area = new Form(*pf,0);


  vector<obstacle*>::iterator it2;
  for (it2=_obs.begin();it2!=_obs.end();it2++) 
	{
  
	  if(GeomFormCollisionDetection(*gripper_area,*((*it2)->give_form())))
		{
		  //Store the pointer to the gripped obstacle
		  gripped_obs.push_back(*it2);
		  
		  //And tag the object as gripped
		  (*it2)->grip();
		}
							 
	}
  
  
  delete gripper_area; //Destroy all

}


void Gripper::release()
{
  vector<obstacle*>::iterator it2;
  for (it2=gripped_obs.begin();it2!=gripped_obs.end();it2++)
	{
	  (*it2)->release();
	}
  
  gripped_obs.clear();
  
}


void Gripper::Translate (Real _tx,Real _ty)
{
  //taking_area->Translate(_tx,_ty);
  
  vector<obstacle*>::iterator it2;
  
  for (it2=gripped_obs.begin();it2!=gripped_obs.end();it2++)
	{
	  	  (*it2)->Translate (_tx,_ty);
	}
  
}
	

void Gripper::Rotate (Real _theta,Real _centerx, Real _centery)
{
  //taking_area->Rotate(_theta,_centerx,_centery);

  vector<obstacle*>::iterator it2;
  
  for (it2=gripped_obs.begin();it2!=gripped_obs.end();it2++)
	{
	  (*it2)->Rotate(_theta,_centerx,_centery);
	}
  
}

	
