#ifndef ROBOT_H
#define ROBOT_H

//#include "world.h" // DO NOT include world.h here (world.h need a robot with display functions ; the include is done after the declaration of robot) 

#include "global.h"
#include "map_controllers.h"
#include <math.h>
//#include "Real.h"

#include "exception.h"
#include <string>
#include <vector>


#include "IRsensor.h" 
#include "../GEOMsources/GeomFormCollisionDetection.h"

#include "GLheader.h"
#include "environment.h"
#include "camera1D.h"
#include "khepera_serial.h"
#include "gripper.h"





#include <iostream>


#define KEEP_FIXED        1
#define KEEP_UNFIXED      2 
#define KEEP_OTHER_ROBOTS 4
#define KEEP_FOOD         8



using namespace std; // for string and gcc-3.2
using namespace Geom;



extern bool display_sensor_around; //defined in global.h
extern bool display_sensor_value;
extern bool show_name_robots;
extern Real cycle_simulation; //seconde


struct position_robot
{
    Real centerx;
    Real centery;
    Real direction;
};



class robot  //Abstract class, all the robots must derive from this class 
// This class MUST stay a virtual pure class to allow dynamic casts
{

 protected:

  class world * wld; //used to put the robot being aware of its environment

  GLint id_robot;
    
  
 public:
  virtual string type()=0;
  virtual void robot_GL3D () =0;
  virtual void robot_GL3D_without_Lists() =0;
  //  virtual void update_weights (vector<double> & _poids)= 0;
  virtual void display_2D () =0;
  virtual void display_3D () =0;
  virtual void display_winSensors (Real shiftx,Real shifty)=0;
  virtual GLubyte* camera2D()=0;
  virtual unsigned* camera1D()=0;
  // virtual void set_robot_according_center_and_direction()=0;
  virtual void Rotate(Real _theta,Real _centerx, Real _centery) =0;
  virtual void Translate(Real _tx,Real _ty) =0;
  virtual void reset_sensors ()=0; //used to reset sensors 
  virtual bool test_collision()=0;
  virtual void update_position()=0;
  virtual void take_a_decision()=0;
  virtual void choose_a_place(Real _dx=-1, Real _dy=-1,Real _direction=0,bool given=false)=0; //used to have a random start in a new world (if the robot have no fixed start) _dx and _dy are the allowed range for the robot
  virtual bool eat()=0;
  virtual vector<Real> proximity_food()=0;
 
 virtual void decide_to_turn_left()=0; // theses func are only used 
  virtual void decide_to_turn_right()=0;// to allow user to take control
  virtual void decide_to_go_forward()=0;// over a robot
  virtual void decide_to_go_back()=0;
  virtual void decide_to_stop()=0;
  virtual void decide_gripper_up()=0;
  virtual void decide_gripper_down()=0;
  virtual void decide_to_grip()=0;
  virtual void decide_to_release()=0;
  virtual void reset_decision()=0;

  //For the REAL khepera
  virtual void real_move()=0;
  
  //Internal 
  Real centerx; // All robots must have a center (used for locomotion)
  Real centery;
  Real direction;

  
  bool crash_obstacle;
  bool crash_obstacle_unfixed;
  bool crash_other_robot;
  bool on_food;
  

  string namerobot;
  
  class Form* Forme_robot_pointeur;
  // vector<Form*> useful_forms_of_environment; // forms that the robot will have to deal  // vector<Form*> useful_forms_of_robots;


  //class Memory* memory;
  //  vector<class Communication*> vector_of_incoming_communications;
  // vector<class Communication*> vector_of_output_communications;
 
  //blackbox blackbox_robot;
  // unsigned constructor;

  
  bool random_start;
  Real startx;
  Real starty;
  Real startdirection;
  int robot_serial;
  
    
  class Form* give_form();  
  void set_world(class world *wl);

  
  
  robot (string _namerobot) 
	{
	  wld=NULL;
	  namerobot=_namerobot;
	}

  void reset_crash_sensors ();
  bool crash();
    

  virtual ~robot () {}

	
};

#include "world.h"



//extern GLint id_robot;
//extern double t;

struct decision_khepera
{
  Real left_motor; //value_between 0.0 and 1.0, between 0.0 and 0.5, the wheel goes backward and between 0.5 and 1.0 the wheel goes forward
  Real right_motor; //value_between 0.0 and 1.0, between 0.0 and 0.5, the wheel goes backward and between 0.5 and 1.0 the wheel goes forward
  bool gripper_take;
  bool gripper_release;
  int  gripper_angle;
 
  
  decision_khepera() 
  {
	left_motor=0.5;
	right_motor=0.5;
	gripper_take=false;
	gripper_release=false;
	gripper_angle=250;
  }
};



class khepera : public robot // Typical khepera robots

{
 private:
    // void (* controller) (khepera &);
  
    GLint id_sensors[8];
	
	IRsensor *IR_[8];
	
	Gripper gripper_;
	
	
	//	eoArchi * archi;
    
	
	GLubyte* camera2D_picture;
	//unsigned short* camera1D_picture;
	Camera_1D* cam1D;
	
 public:

	Controller * controller;
	bool have_a_1D_cam;
	bool have_a_2D_cam;



    decision_khepera decision;
	Real left_motor_speed;
	Real right_motor_speed;
	Real battery; //Simulation of Khepera battery
	
	void add_2D_cam();
	virtual GLubyte* camera2D();
	
	void add_1D_cam();
	virtual unsigned* camera1D();
	
	
    //Real speed; //The speed is between -10.0 and 10.0
	void IRplace_from_center(const Real radius,const int nthsensor,Real reply[3] );
	void IRplace(Real radius);
	void IRinit(Real radius);
	//	virtual void update_weights (vector<double> & _poids);
	  
	
	khepera(string _namerobot, Controller * _controller_base, Real _radius=KHEPERA_DEFAULT_RADIUS) : robot(_namerobot) 
	  {
		// initialisation with a random center
		// Definition of khepera form : circle => forme_primitive => Form

		controller = _controller_base;
		//constructor = 0;
		
		centerx=0.0;
		centery=0.0;

		Circle2 *cir = new Circle2;
		cir->Center()=Vector2(centerx,centery);
		cir->Radius()=_radius;

		vector<PrimitivForm *>*pf=new vector <PrimitivForm *> ;
		pf->push_back(cir);

		Forme_robot_pointeur=new Form(*pf);
	  
		// other initialisations
		direction=((Real)rand())/RAND_MAX;
		crash_obstacle=false;
		crash_other_robot=false;
		random_start=true;
		have_a_2D_cam=false;
		have_a_1D_cam=false;

		// IRsensor initialisations 
		IRinit(cir->Radius());

		//GL_initialisation
		robot_GL3D();

	  };
	

    khepera(string _namerobot, Controller * _controller_base, Real _centerx, Real _centery, Real _direction, Real _radius=KHEPERA_DEFAULT_RADIUS) : robot(_namerobot) 
	  {
		// Definition of khepera form : circle => forme_primitive => Form
	  
		//constructor = 0;
		
		controller = _controller_base;
		
	    centerx=_centerx;
		centery=_centery;
	  
		Circle2 *cir =new Circle2;
		cir->Center()=Vector2(_centerx,_centery);
		cir->Radius()=_radius;

		vector<PrimitivForm *> *pf=new vector <PrimitivForm *> ;
		pf->push_back(cir);

		Forme_robot_pointeur=new Form(*pf);
	  
		// other initialisations
		direction=_direction;
		crash_obstacle=false;
		crash_other_robot=false;
		random_start=false;
		startx=_centerx;
		starty=_centery;
		startdirection=_direction;
		have_a_2D_cam=false;
		have_a_1D_cam=false;

		
		//sensor initialisation
		IRinit(cir->Radius());
	  
		//GL_initialisation
		robot_GL3D();
	  }
	


    virtual ~khepera()
	  {
		Partial_self_destruct();
	  };


	virtual string type();

	//initialize the GL representation of the robot
    virtual void robot_GL3D();
	
	virtual void robot_GL3D_without_Lists();
	
    virtual bool test_collision();
	
	
    virtual void update_position();
	 	
	//OpenGL Procedures for drawing robot

	
    virtual void display_2D ();
	
    virtual void display_3D ();

    virtual void display_winSensors(Real shiftx,Real shifty);
		

	//used after giving a new center and direction to have a coherent robot
	//This procedure is EVIL ... but it's also the fastest... the good way is to use Rotate and Translate methods of robot
    void set_robot_according_center_and_direction();
	
	
    virtual void Rotate(Real _theta,Real _centerx, Real _centery); 
	
    virtual void Translate(Real _tx,Real _ty) ;
	
    virtual void reset_sensors ();
		
	
    virtual void take_a_decision();

    virtual void choose_a_place(Real _dx=-1, Real _dy=-1,Real _direction=0,bool given=false);
	 
    virtual bool eat();
    virtual vector<Real> proximity_food();

	void verify_decision();
	
    virtual void decide_to_turn_left();

    virtual void decide_to_turn_right();

    virtual void decide_to_go_forward();

    virtual void decide_to_go_back();
    
    virtual void decide_to_stop();

	virtual void decide_gripper_up();
	
	virtual void decide_gripper_down();

	virtual void decide_to_grip();

	virtual void decide_to_release();

	virtual void reset_decision();
	
	vector<Form*> find_forms(unsigned keep);
	
    void manage_gripper_khepera();
	
    // warning an IR sensor is computed only one time (at the first acces) to have a new computation you have to reset it. This point is useful to avoid to recalc sensor when you acces 2 or more times in the controller.
    Real IR(int i);
	
    Real IR_passive(int i);
	 	
	virtual void real_move();

	void Partial_self_destruct();
	
	
};

#endif

