#ifndef _controller_sample_h
#define _controller_sample_h



#include "global.h"
#include "robot.h"
#include "list_controllers.h"
#include <iostream>


#include "khepera_EO.h"

void controller_with_blackbox(robot & _myrobot) 
 
{
#ifdef _WIN32
    khepera_EO * myrobot =(static_cast<khepera_EO *>(& _myrobot));
#else
    khepera_EO * myrobot =(dynamic_cast<khepera_EO *>(& _myrobot));
#endif
	
    vector<Real> inputs(8);

    for (unsigned i=0; i<8;i++)
	{
			 
	    inputs[i] = myrobot->IR(i)/KHEPERA_IR_Intensity;
	}

    vector<Real> outputs;
	
    outputs = (myrobot->blackbox_robot_EO)(inputs);

    myrobot->decision.left_motor=outputs[0];

	myrobot->decision.right_motor=outputs[1]; // the robot goes forward at maximal speed 

}



//This function will be used for using (to load a blacbox from) an already evolved controller
blackbox_EO * set_blackbox (string _controller_to_load)

{
 		    
    eoState statetmp;
    
    eoPop <Indi > pop;
    statetmp.registerObject(pop);
    
    statetmp.load(_controller_to_load);

    Indi * temp;

    temp = new Indi ( pop.best_element() );

    blackbox_EO * blackbox_allocated;

    blackbox_allocated = new blackbox_EO();

    blackbox_allocated->update_weights((temp->robots[0]).poids);

    blackbox_allocated->update_archi((temp->robots[0]).archi);

    return blackbox_allocated;

}


static blackbox_EO * blackbox_controller_save;
static bool first_time_blackbox_robot_controller_save=true;

void controller_save(robot & _myrobot)
{
#ifdef _WIN32
  // khepera_EO * myrobot =(static_cast<khepera_EO *>(& _myrobot));
#else
	//    khepera_EO * myrobot =(dynamic_cast<khepera_EO *>(& _myrobot));
#endif

    if (first_time_blackbox_robot_controller_save) 
	{
	    blackbox_controller_save=set_blackbox("Save/backprop1.sav");
	    first_time_blackbox_robot_controller_save=false;
	}
    controller_with_blackbox(_myrobot);

}


void simple_supervisor(robot & _myrobot) 
 
{
#ifdef _WIN32
    khepera_EO * myrobot =(static_cast<khepera_EO *>(& _myrobot));
#else
    khepera_EO * myrobot =(dynamic_cast<khepera_EO *>(& _myrobot));
#endif
	
    vector<Real> inputs(8);

    for (unsigned i=0; i<8;i++)
	{
			 
	    inputs[i] = myrobot->IR(i)/KHEPERA_IR_Intensity;
	}

    vector<Real> outputs;
	
    outputs = (myrobot->blackbox_robot_EO)(inputs);
	  
    Real max=-1.0;

    int controller_choosen=0;
	  
    for (unsigned i=0; i< outputs.size();i++)
	{
	    if (outputs[i]>max) controller_choosen=i;
	}

    //Controller * test;
	  
    //string altest="controller_save";

    //test = all_the_controllers("controller_save");
	  

    if (controller_choosen==0) (*(all_the_controllers("controller_save")))(_myrobot);

}


void symbolic_function (vector<double> & _outputs, Real & _mot0, Real & _mot1)
{
    //    cout<< "symbolic_function_begin" <<endl;

    Real max=-1.0;
    
    int behaviour=0;
    
    unsigned nbBehaviour=3;  //Moving Forward, turning left, turning right
    
    for (unsigned i=0; i< nbBehaviour;i++)
	{
	    if (_outputs[i]>max) {
		behaviour=i;
		max=_outputs[i];
	    }
	    
	}
    
      
    switch(behaviour){
    case 0: _mot0= .5 * (_outputs[nbBehaviour] + 1); // forward, 
	_mot1= .5 * (_outputs[nbBehaviour] + 1);
	break;
	
    case 1: _mot0= 1.0-(.5 * (_outputs[nbBehaviour+1] + 1)) ; // turn left  //anticlockwise
	_mot1= .5 * (_outputs[nbBehaviour+1] + 1);
	break;
	
    case 2: _mot0= .5 * (_outputs[nbBehaviour+2] + 1); // turn right   //clockwise
	_mot1= 1.0-(.5 * (_outputs[nbBehaviour+2] + 1));
	break;
	
	//case 3: mot0= .5 * (1 - output[nbBehaviour+3]);// backward 
	//mot1= .5 * (1 - output[nbBehaviour+3]);
	//break;
	
    default: throw breakerror("There seems to be something wrong in setBehaviour ;-)");
    }
    
    //cout << "symbolic_function_end" <<endl;

}


//Now this is controller_battery with 6 outputs, the symbolic outputs

void controller_battery(robot & _myrobot) 
 
{
    //cout << "controller_battery_begin" <<endl;

#ifdef _WIN32
    khepera_EO * myrobot =(static_cast<khepera_EO *>(& _myrobot));
#else
    khepera_EO * myrobot =(dynamic_cast<khepera_EO *>(& _myrobot));
#endif
	
    vector<Real> inputs(17);

    for (unsigned i=0; i<8;i++)
	{
			 
	    inputs[i] = myrobot->IR(i)/KHEPERA_IR_Intensity;
	}

    //    cout << "light level ";

    for (unsigned i=8; i<16;i++)
	{
			 
	    inputs[i] = myrobot->IR_passive(i-8)/KHEPERA_passive_IR_Intensity;
	    //    cout << inputs[i] << " ";
	}

    //  cout << endl;
    
    inputs[16]=myrobot->battery;

    vector<Real> outputs;
	
    outputs = (myrobot->blackbox_robot_EO)(inputs);

    Real mot0,mot1;

    symbolic_function(outputs,mot0,mot1);

    myrobot->decision.left_motor=mot0;
 
    myrobot->decision.right_motor=mot1; // the robot goes forward at maximal speed 

    //cout << "controller_battery_end" <<endl;

}



void symbolic_controller (robot & _myrobot) 
 
{


#ifdef _WIN32
    khepera_EO * myrobot =(static_cast<khepera_EO *>(& _myrobot));
#else
    khepera_EO * myrobot =(dynamic_cast<khepera_EO *>(& _myrobot));
#endif

    vector<Real> inputs(8);
	  
    for (unsigned i=0; i<8;i++)
	{
		  
	    inputs[i] = myrobot->IR(i);
	}
	  
    vector<Real> outputs;
	  
    outputs = (myrobot->blackbox_robot_EO)(inputs);
	  
    Real mot0,mot1;

    symbolic_function(outputs,mot0,mot1); 

    myrobot->decision.left_motor=mot0;

    myrobot->decision.right_motor=mot1; 



}



#endif
