/** -*- mode: c++; c-indent-level: 4; c++-member-init-indent: 8; comment-column: 35; -*-

The above line is usefulin Emacs-like editors
*/




#ifndef _eoArchi_h
#define _eoArchi_h

#include "exception.h"

//#include "eoRN.h"

double trigger(double entree, double oldsortie);
double sigmoid(double entree);
double sigmoid_alpha(double entree,double alpha);

/* Classe de base pour les archis */

class eoRNArchiBase
{
public:
  // virtual void calcul(vector<double> poids, vector<double> _v) = 0;
  virtual void calcul(vector<double> & poids, vector<double> & _v, vector<double> & _sortie) = 0;

  virtual unsigned NbPoids() = 0;
};

/** description of a general NN architecture */
class eoArchi : public eoRNArchiBase 
{
public:
  eoArchi(){}

  eoArchi(unsigned _nbEntree, unsigned _nbSortie, unsigned _nbNeurone):
    nbEntree(_nbEntree), nbSortie(_nbSortie),nbNeurone(_nbNeurone)
  {}
 
  eoArchi(istream & _is)
  {
    throw breakerror("Pas encore implemante");
  }

  virtual ~eoArchi(){}

  /*
    virtual void setValues(unsigned _e, unsigned _s, unsigned _n, unsigned _c)
    {
    nbEntree = _e;
    nbSortie = _s;
    nbNeurone = _n;
    nbConnection = _c;

    //	    connectivite.resize(nbNeurone*nbNeurone);
    // for (unsigned i=0; i<nbNeurone*nbNeurone; i++)
    //connectivite[i]=0;
    }
  */

  virtual string className() {return "GENERAL";}

  virtual void printOn(ostream & _os)
  {
    _os << className() << " " ;
    _os <<  nbEntree << " " ;
    _os <<  nbSortie << " ";
    _os << nbNeurone << " ";
    _os << nbConnection << " " ;
  }

  virtual unsigned NbPoids() { return nbConnection;}
  virtual void erase_recurrent () {};
  
  virtual void calcul(vector<double> & poids, vector<double> & _v, vector<double> & _sortie)=0;

  // accessors
  unsigned NbEntree() {return nbEntree;}
  unsigned NbSortie() {return nbSortie;}
  bool erase_each_epoch;


protected:
  unsigned nbEntree;
  unsigned nbSortie;
  unsigned nbNeurone;
  unsigned nbConnection;
};

/** une specialistion; les perceptron multi-couche */
class eoMultiLayer : public eoArchi
{
public:
  /** Ctor from parameters (e.g. as read from the parser) */
  eoMultiLayer(vector<unsigned> & _lesC): lesCouches(_lesC), debutCouche(_lesC.size()+1)
  {
    majCtor();
  }
  
  /** Ctor from an istream - uses readFrom */
  eoMultiLayer(istream & _is)
  {
    readFrom(_is);
  }

  virtual ~eoMultiLayer(){}
  
  virtual string className() {return "MULTILAYER";}

  virtual void printOn(ostream & _os)
  {
    _os << className() << " " ;
    _os <<  lesCouches.size() << " " ;
    for (unsigned i=0; i<lesCouches.size(); i++)
      _os <<  lesCouches[i] << " ";
  }
  
  virtual void readFrom(istream & _is)
  {

    
    unsigned nbCouche;
    
    _is >> nbCouche;
    
    lesCouches.resize(nbCouche);
    
    debutCouche.resize(nbCouche+1);
    

    for (unsigned i=0; i<nbCouche; i++)
      _is >> lesCouches[i];

    
    majCtor();
    
  }
  
  virtual void calcul(vector<double> & poids, vector<double> & _v, 
		      vector<double> & _sortie)
  {
    unsigned i,j,k;
    
    vector<unsigned> & structu = lesCouches;
    vector<double> activation(nbNeurone);

    // copy of input layer
    for (unsigned  i=0;i<structu[0];i++)
      {
	activation[i]=_v[i];
	//	   if (activation[i] > 1.0) activation[i] = 1.0;
	//	   else if (activation[i] <0.0) activation[i] =0.0;
      }
    
    unsigned index=0;		   // index of current weight (vector poids)
    
    for (k=1;k<structu.size();k++)  //the first layer is already activated
      {
	for (i=debutCouche[k]; i<debutCouche[k+1]; i++) // neurons in current layer
	  {
	    double sss=poids[index++]; // bias
	    for (j=debutCouche[k-1]; j< debutCouche[k]; j++) // from neurons of previous layer
	      {
		sss +=  poids[index++]*activation[j];
	      }
	    activation[i]=sigmoid(sss);
	  }
      }
    
    _sortie.resize(nbSortie);

    for (i=nbNeurone-nbSortie, j=0; i<nbNeurone; i++,j++)
      {
	_sortie[j] = activation[i];
      }
    /*
      if (numero ==1)
      output = _sortie;
    */
    // return _sortie;  
  }

  /////////////////////////// private data and methods /////////////////////////////
private:
  vector<unsigned> lesCouches;
  vector<unsigned> debutCouche;   // helper for calcul
  
  /** Mise a Jour (nbNeurones, nbSorties, ...) apres lectures de couches */
  void majCtor()
  {
    unsigned nbCouche = lesCouches.size();
    nbEntree = lesCouches[0];
    nbSortie = lesCouches[nbCouche-1];
    // on compte les neurones et les connextions, on remplit debutCouche
    debutCouche[0]=0;
    nbNeurone = lesCouches[0];
    nbConnection = 0;
    for (unsigned i=1; i<nbCouche; i++)
      {
	debutCouche[i] = nbNeurone;
	nbNeurone += lesCouches[i];
	nbConnection += lesCouches[i] *(lesCouches[i-1]+1);
      }
    debutCouche[nbCouche] = nbNeurone; // total  number of neurons == end of last layer
  }
  
};


/** une specialisation; les perceptron FeedForward */


class eoFeedForward : public eoArchi
{
public:
  /** Ctor from parameters (e.g. as read from the parser) */
  eoFeedForward(unsigned _nbEntree, unsigned _nbCachee, 
		unsigned _nbSortie, vector<bool> _connectivite) :
    eoArchi(_nbEntree, _nbSortie, _nbCachee+_nbEntree+_nbSortie), 
    connectivite(_connectivite)
  { 
    // attention, il faut  rajouter les biais
    nbConnection=_nbCachee+_nbSortie;
    for (unsigned i=0; i<connectivite.size(); i++)
      if (connectivite[i])
	nbConnection++;
  }

  /** Ctor from an istream - uses readFrom */
  eoFeedForward(istream & _is)
  {
    readFrom(_is);
  }

  virtual ~eoFeedForward(){}

  virtual string className() {return "FEEDFORWARD";}

  // accessors et setters
  void setConnection(unsigned i, unsigned j)
  {
    connectivite[i+j*nbNeurone]=true;
  }

  bool getConnection(unsigned i, unsigned j)
  {
    return connectivite[i+j*nbNeurone];
  }

  /** printOn: parcimonious printing */
  virtual void printOn(ostream & _os)
  {
    unsigned int nbCachee = nbNeurone - (nbEntree+nbSortie);
    _os << className() << " " << nbEntree << " " 
	<< nbCachee << " " << nbSortie << " " ;
    // La connectivite - write only non-zero terms. 
    // write their number first = nb of weights - connections to the biases
    unsigned sizeConnect = NbPoids() - (nbCachee + nbSortie);
    _os << sizeConnect << " ";
    for (unsigned i=0; i<nbNeurone*nbNeurone; i++)
      if (connectivite[i])
	_os << i << " ";
  }

  /** and reverse reading, except we don't read the className */
  virtual void readFrom(istream & _is)
  {
    unsigned int nbCachee;
    _is >> nbEntree;
    _is >> nbCachee;
    _is >> nbSortie;
    nbNeurone = nbEntree + nbCachee + nbSortie;

    // allocate memory for connectivity matrix
    connectivite.resize(nbNeurone*nbNeurone);
    // just to be sure
    for (vector<bool>::iterator it=connectivite.begin(); 
	 it < connectivite.end(); it++)
      (*it)=false;

    // Read connectivity - only non-zero terms are written
    // their number first = nb of weights - connections to the biases
    unsigned sizeConnect;
    _is >> sizeConnect;
    // add the biases
    nbConnection = sizeConnect+ nbCachee + nbSortie;
    // and read which connections
    for (unsigned i=0; i<sizeConnect; i++)
      {
	unsigned which;
	_is >> which;
	connectivite[which]=true;
      }
  }

  virtual void calcul(vector<double> & poids, vector<double> & _v, vector<double> & _sortie)
  {
    unsigned i;
    vector<double> activation(nbNeurone);

    for (i=0;i<nbEntree;i++)
      {
	activation[i]=_v[i];
      }
    unsigned index=0;
    for (i=nbEntree; i<nbNeurone; i++)
      {
	double sss = poids[index++];
	vector<bool>::iterator it = connectivite.begin() + i * nbNeurone;
	for (unsigned j=0; j<i; j++)
	  {
	    if (*it)
	      sss += poids[index++] * activation[j];
	    it++;
	  }
	activation[i] = sigmoid(sss);
      }    

    _sortie.resize(nbSortie);

    for (i=0;i<nbSortie;i++)
      {
	_sortie[i] = activation[i + nbNeurone - nbSortie]; //activation[i + nbConn - 1 - nbSortie];
      }


  }
      
private:
  vector<bool> connectivite;
};

/** une specialisation; the recurrent networks */


class eoRecurrentHidden : public eoArchi //for recurrency in 
{
public:
  /** Ctor from parameters (e.g. as read from the parser) */
  
 

  eoRecurrentHidden (vector<unsigned> & _lesC): lesCouches(_lesC), 
						debutCouche(_lesC.size()+1)
  {
    majCtor();
  }
  
  /** Ctor from an istream - uses readFrom */
  eoRecurrentHidden(istream & _is)
  {
    readFrom(_is);
  }

  virtual ~eoRecurrentHidden(){}
  
  virtual string className() {return "ELMAN";}

  virtual void printOn(ostream & _os)
  {
    _os << className() << " " ;
    _os <<  lesCouches.size() << " " ;
    for (unsigned i=0; i<lesCouches.size(); i++)
      _os <<  lesCouches[i] << " ";
   
  }
  
  virtual void readFrom(istream & _is)
  {
    unsigned nbCouche;
    _is >> nbCouche;
    lesCouches.resize(nbCouche);
    debutCouche.resize(nbCouche+1);

    for (unsigned i=0; i<nbCouche; i++)
      _is >> lesCouches[i];
    majCtor();
  }


  virtual void erase_recurrent ()
  {


    if (erase_each_epoch)
      {
	
	
        for (unsigned i=0; i<recurrenthidden.size(); i++)
	  { 
	    recurrenthidden[i]=0.0;
	  }
      }
  }

  
  virtual void calcul(vector<double> & poids, vector<double> & _v, 
		      vector<double> & _sortie)
  {
    unsigned i,j,k;
    
    vector<unsigned> & structu = lesCouches;
    vector<double> activation(nbNeurone);
    recurrenthidden.resize(nbNeurone);


    // copy of input layer
    for (unsigned  i=0;i<structu[0];i++)
      {
	activation[i]=_v[i];
	recurrenthidden[i]=_v[i];
	
      }
    
    unsigned index=0;		   // index of current weight (vector poids)
    
    for (k=1;k<structu.size();k++)  //the first layer is already activated
      {
	for (i=debutCouche[k]; i<debutCouche[k+1]; i++) // neurons in current layer
	  {
	    double sss=poids[index++]; // bias
	    double sss2=sss;

	    
	    if (k!=structu.size()-1)  //No recurrent connections on the output neurons
	      for (unsigned layinc=debutCouche[k]; layinc<debutCouche[k+1]; layinc++)
		sss2 += poids[index++]*recurrenthidden[layinc];
	    
	    
	    
	    for (j=debutCouche[k-1]; j< debutCouche[k]; j++) // from neurons of previous layer
	      {
		
		sss2 += poids[index++]*recurrenthidden[j];
		
		

	      }
	    activation[i]=sigmoid(sss2);
	  }
      }
    for(unsigned  i=0;i<nbNeurone;i++)
      {
	recurrenthidden[i]=activation[i];
      }
    _sortie.resize(nbSortie);

    for (i=nbNeurone-nbSortie, j=0; i<nbNeurone; i++,j++)
      {
	_sortie[j] = activation[i];
      }
    /*
      if (numero ==1)
      output = _sortie;
    */
    // return _sortie;  
  }

  /////////////////////////// private data and methods /////////////////////////////
private:
  vector<unsigned> lesCouches;
  vector<unsigned> debutCouche;   // helper for calcul
  vector<double> recurrenthidden; 
  /** Mise a Jour (nbNeurones, nbSorties, ...) apres lectures de couches */
  void majCtor()
  {
    unsigned nbCouche = lesCouches.size();
    nbEntree = lesCouches[0];
    nbSortie = lesCouches[nbCouche-1];
    // on compte les neurones et les connextions, on remplit debutCouche
    debutCouche[0]=0;
    nbNeurone = lesCouches[0];
    nbConnection = 0;
    for (unsigned i=1; i<nbCouche; i++)
      {
	//	if (i==2)
	//  debutCouche[i] = nbNeurone+lesCouches[1]-2;
	//	else
	debutCouche[i] = nbNeurone;

	nbNeurone += lesCouches[i];
	nbConnection += lesCouches[i] *(lesCouches[i-1]+1);  //we add the numbers of recurrent connections for the layer i + bias
	
	if (i!=nbCouche-1) //not for the output layer
	  nbConnection += lesCouches[i]*lesCouches[i];  //recurrent connection on the layer (Elman 1990)
	
      }

    
    recurrenthidden.resize(nbNeurone);
    for (unsigned i=0; i<recurrenthidden.size(); i++)
      { 
	recurrenthidden[i]=0.0;
      }
    debutCouche[nbCouche] = nbNeurone; // total  number of neurons == end of last layer
  }
  
};

//New Class

class eoRecurrentHiddenTrigger : public eoArchi // EXPERIMENTAL NETWORK ...  trigger neurons
{
public:
 

  /** Ctor from parameters (e.g. as read from the parser) */
  eoRecurrentHiddenTrigger (vector<unsigned> & _lesC): lesCouches(_lesC), 
						       debutCouche(_lesC.size()+1)
  {
    majCtor();
  }
  
  /** Ctor from an istream - uses readFrom */
  eoRecurrentHiddenTrigger(istream & _is)
  {
    readFrom(_is);
  }

  virtual ~eoRecurrentHiddenTrigger(){}
  
  virtual string className() {return "RECURRENTHIDDENTRIGGER";}

  virtual void printOn(ostream & _os)
  {
    _os << className() << " " ;
    _os <<  lesCouches.size() << " " ;
    for (unsigned i=0; i<lesCouches.size(); i++)
      _os <<  lesCouches[i] << " ";
   
  }
  
  virtual void readFrom(istream & _is)
  {
    unsigned nbCouche;
    _is >> nbCouche;
    lesCouches.resize(nbCouche);
    debutCouche.resize(nbCouche+1);

    for (unsigned i=0; i<nbCouche; i++)
      _is >> lesCouches[i];
    majCtor();
  }
  

  virtual void erase_recurrent ()
  {
    if (erase_each_epoch)
      for (unsigned i=0; i<recurrenthidden.size(); i++)
	{ 
	  recurrenthidden[i]=0.0;
	}
  }

  virtual void calcul(vector<double> & poids, vector<double> & _v, 
		      vector<double> & _sortie)
  {
    unsigned i,j,k;
    
    vector<unsigned> & structu = lesCouches;
    vector<double> activation(nbNeurone);
    recurrenthidden.resize(nbNeurone);

    // copy of input layer
    for (unsigned  i=0;i<structu[0];i++)
      {
	activation[i]=_v[i];
	recurrenthidden[i]=_v[i];
	//	   if (activation[i] > 1.0) activation[i] = 1.0;
	//	   else if (activation[i] <0.0) activation[i] =0.0;
      }
    
    unsigned index=0;		   // index of current weight (vector poids)
    
    for (k=1;k<structu.size();k++)  //the first layer is already activated
      {
	for (i=debutCouche[k]; i<debutCouche[k+1]; i++) // neurons in current layer
	  {
	    double sss=poids[index++]; // bias
	    double sss2=sss;

	    if ((k<structu.size()-1) && (i==debutCouche[k]))
	      sss2 += poids[index++]*recurrenthidden[i+1];  //for the recurrence at the beginning of the hidden layer k
	    
	    if (k<structu.size()-1) //not for the first layer and the last layer (output), only for the hidden layer
	      sss2 += poids[index++]*recurrenthidden[i];  //for the reccurence on the neuron itself

	    if ((k<structu.size()-1) && (i>debutCouche[k]) && (i<debutCouche[k+1]-1))
	      {
		sss2 += poids[index++]*recurrenthidden[i+1];
		sss2 += poids[index++]*recurrenthidden[i-1];
	      }
	    
	    if ((k<structu.size()-1) && (i==(debutCouche[k+1]-1)))
	      sss2 += poids[index++]*recurrenthidden[i-1];
	    
	    if (k==1)
	      for (j=debutCouche[k-1]; j< debutCouche[k]; j++) // from neurons of previous layer
		{
		
		  sss2 += poids[index++]*recurrenthidden[j];
		
		}
	    
	    if (k!=1 && (k%2)!=0)
	      for (j=debutCouche[k-2]; j< debutCouche[k-1]; j++) // from neurons of previous hidden layer or fcr the output layer
		{
		
		  sss2 += poids[index++]*recurrenthidden[j];
		
		}

	    if ((k<structu.size()-1) && (k%2)!=0)
	      for (j=debutCouche[k+1]; j< debutCouche[k+2]; j++) // for the links between the hidden and the memory neurons
		{
		
		  sss2 += poids[index++]*recurrenthidden[j];
		
		}

	    if ((k<structu.size()-1) && (k%2)==0)
	      for (j=debutCouche[k-1]; j< debutCouche[k]; j++) // for the links between the memory and the hidden neurons
		{
		
		  sss2 += poids[index++]*recurrenthidden[j];
		
		}


	    if ((k%2)!=0)    activation[i]=sigmoid(sss2);
	    if ((k%2)!=0)    activation[i]=trigger(sss2,recurrenthidden[i]);
	  }
      }
    for(unsigned  i=0;i<nbNeurone;i++)
      {
	recurrenthidden[i]=activation[i];
      }
    _sortie.resize(nbSortie);

    for (i=nbNeurone-nbSortie, j=0; i<nbNeurone; i++,j++)
      {
	_sortie[j] = activation[i];
      }
    /*
      if (numero ==1)
      output = _sortie;
    */
    // return _sortie;  
  }

  /////////////////////////// private data and methods /////////////////////////////
private:
  vector<unsigned> lesCouches;
  vector<unsigned> debutCouche;   // helper for calcul
  vector<double> recurrenthidden; 
  /** Mise a Jour (nbNeurones, nbSorties, ...) apres lectures de couches */
  void majCtor()
  {
    unsigned nbCouche = lesCouches.size();
    nbEntree = lesCouches[0];
    nbSortie = lesCouches[nbCouche-1];
    // on compte les neurones et les connextions, on remplit debutCouche
    debutCouche[0]=0;
    nbNeurone = lesCouches[0];
    nbConnection = 0;
    for (unsigned i=1; i<nbCouche; i++)
      {
	//	if (i==2)
	//  debutCouche[i] = nbNeurone+lesCouches[1]-2;
	//	else
	debutCouche[i] = nbNeurone;

	nbNeurone += lesCouches[i];  // computation of the number of neurons
	nbConnection += lesCouches[i]; // biases

	if ((i%2)!=0 && i==1)	//not for memory neurons and for layer 1
	  nbConnection += lesCouches[i] *(lesCouches[i-1]);  


	if ((i%2)!=0 && i!=1)	//not for memory neurons and not for layer 1
	  nbConnection += lesCouches[i] *(lesCouches[i-2]);  

	if (i!=(nbCouche-1) && (i%2)!=0)
	  nbConnection += lesCouches[i]*2-2+lesCouches[i]; //we add the numbers of recurrent connections for the hidden layer i

	if (i!=(nbCouche-1) && (i%2)==0)
	  nbConnection += lesCouches[i]*2-2+lesCouches[i]; //we add the numbers of recurrent connections for the memory layer i


	if (i!=(nbCouche-1) && (i%2)==0)
	  nbConnection += lesCouches[i-1]*lesCouches[i]*2;  //we add the weights relying the memory neurons to the hidden neurons

      }

    
    recurrenthidden.resize(nbNeurone);
    for (unsigned i=0; i<recurrenthidden.size(); i++)
      { 
	recurrenthidden[i]=0.0;
      }
    debutCouche[nbCouche] = nbNeurone; // total  number of neurons == end of last layer
  }
  
};




/*! 
 *  \brief Architecture of anticipation, simple version, the weights are modified by Artificial Evolution
 * 
 *
 *   Sample of a neural network using this archi:
 *   
 *   First part of the network:
 *
 *
 *   O----
 *   O-----                    O----
 *   O------                   O-----
 *   O-------                  O------                O
 *   O-------                  O------                O 
 *   O------                   O-----
 *   O-----                    O----
 *   O----
 * First Layer (L1)           Second Layer (L2)           Last Layer (L3)
 * (sensors activitites)   (hidden Layer)       (Output neurons - typically the motors activations)
 *
 *   Fully network
 *
 *   
 *   L1 ------->  L2  -----------> L3
 *          >      |                |    
 *          |      |                |
 *          |      ------------------
 *          |              ||
 *          |             \ /
 *   O      |           O O O O O   (Hidden layer of the second network) (L4)
 *   O      |              |
 *   O      |              |
 *   O -----|              |
 *   O                    \/ 
 *   O (L6)       <--  O O O O O O O O (Prediction of the sensor activation at t+1) (L5)        
 *   O      (we put the deltas between the prediction of the activtion at t+1 and the real senations obtained at t+1) 
 *   O     
 * Let's note that this network is ELMAN type, i.e. the hidden layer is fully recurrent 
 *
 *
 *  So the fully network is in fact :
 *
 *   L1 ----/
 *          / ---  L2  ------> L3
 *   L6 ----/       |           |
 *                  |           |
 *                  -------------
 *                        |  
 *                       L4
 *                        |
 *                       L5 
 *
 *  \author Nicolas Godzik
 *  \version 1.0
 *  \date    October 2003
 *  \bug No
 *  \warning 
 */



class AnticipationNetwork : public eoArchi //for recurrency in 
{
public:


  /** Ctor from parameters (e.g. as read from the parser) */
  AnticipationNetwork (vector<unsigned> & _lesC): lesCouchesProvi(_lesC)//, debutCouche(_lesC.size()+1)
  {
    majCtor();
  }
  
  /** Ctor from an istream - uses readFrom */
  AnticipationNetwork(istream & _is)
  {
    readFrom(_is);
  }

  virtual ~AnticipationNetwork(){}
  
  virtual string className() {return "AnticipationNetwork";}

  virtual void printOn(ostream & _os)
  {
    _os << className() << " " ;
    _os <<  lesCouchesProvi.size() << " " ;
    for (unsigned i=0; i<lesCouchesProvi.size(); i++)
      _os <<  lesCouchesProvi[i] << " ";
   
  }
  
  virtual void readFrom(istream & _is)
  {
    unsigned nbCoucheProvi;
    _is >> nbCoucheProvi;
    lesCouchesProvi.resize(nbCoucheProvi);
    //debutCoucheNetwork1.resize(nbCouche+1);

    for (unsigned i=0; i<nbCoucheProvi; i++)
      _is >> lesCouchesProvi[i];
    majCtor();
  }
  
  virtual void erase_recurrent ()
  {
    if (erase_each_epoch)
      {
        for (unsigned i=0; i<recurrenthidden.size(); i++)
	  { 
	    recurrenthidden[i]=0.0;
	  }
	for (unsigned i=0; i<recurrenthidden2.size(); i++)
	  { 
	    recurrenthidden2[i]=0.0;
	  }
      }
  }

  virtual void calcul(vector<double> & poids, vector<double> & _v, 
		      vector<double> & _sortie)
  {
    unsigned i,j,k;
    
    vector<unsigned> & structu = lesCouches;
    vector<double> activation(nbNeuroneNetwork1);
    recurrenthidden.resize(nbNeuroneNetwork1);
    recurrenthidden2.resize(nbNeuroneNetwork2);

    _v.resize(lesCouches[0]*2);
    
    savediff.resize(lesCouches[0]);

    for (unsigned  i=0;i<structu[0];i++)
      {
	_v[structu[0]+i]=fabs(recurrenthidden2[nbNeuroneNetwork2-structu[0]+i]-_v[i]);  //Add delta to the input of the network1
	savediff[i]=_v[structu[0]+i];
	//	   if (activation[i] > 1.0) activation[i] = 1.0;
	//	   else if (activation[i] <0.0) activation[i] =0.0;
      }


   

    // copy of input layer
    for (unsigned  i=0;i<structu[0]*2;i++)
      {
	activation[i]=_v[i];
	recurrenthidden[i]=_v[i];
	//	   if (activation[i] > 1.0) activation[i] = 1.0;
	//	   else if (activation[i] <0.0) activation[i] =0.0;
      }
    
    unsigned index=0;		   // index of current weight (vector poids)
    
    for (k=1;k<structu.size();k++)  //the first layer is already activated
      {
	for (i=debutCoucheNetwork1[k]; i<debutCoucheNetwork1[k+1]; i++) // neurons in current layer
	  {
	    double sss=poids[index++]; // bias
	    double sss2=sss;

	    
	    for (j=debutCoucheNetwork1[k-1]; j< debutCoucheNetwork1[k]; j++) // from neurons of previous layer
	      {
		
		sss2 += poids[index++]*recurrenthidden[j];
		
	      }
	    
	    if (k!=structu.size()-1)  //No recurrent connections on the output neurons
	      for (unsigned layinc=debutCoucheNetwork1[k]; layinc<debutCoucheNetwork1[k+1]; layinc++)
		sss2 += poids[index++]*recurrenthidden[layinc];
	    	    
	    activation[i]=sigmoid(sss2);
	  }
      }
   


    //And now for the network2


    for(unsigned  i=0;i<nbNeuroneNetwork1;i++)
      {
	recurrenthidden[i]=activation[i];
      }
    _sortie.resize(nbSortieNetwork1);

    for (i=nbNeuroneNetwork1-nbSortieNetwork1, j=0; i<nbNeuroneNetwork1; i++,j++)
      {
	_sortie[j] = activation[i];
      }

    vector<unsigned> & structu2 = lesCouches2;
    vector<double> activation2(nbNeuroneNetwork2);
    recurrenthidden2.resize(nbNeuroneNetwork2);

    vector<double> _v2;

    _v2.resize(structu2[0]);
  
    for (int i=0;i<int(lesCouchesProvi[nbCoucheNetwork1-1]);i++)
	{
	_v2[i]=recurrenthidden[nbNeuroneNetwork1-lesCouchesProvi[nbCoucheNetwork1]-lesCouchesProvi[nbCoucheNetwork1-1]+i];
	}
	
      for (int i=0;i<int(lesCouchesProvi[nbCoucheNetwork1]);i++)
	{
	_v2[lesCouchesProvi[nbCoucheNetwork1-1]+i]=recurrenthidden[nbNeuroneNetwork1-lesCouchesProvi[nbCoucheNetwork1]+i];
	}

    // copy of input layer

    
    for (unsigned  i=0;i<structu2[0];i++)
      {
	
	activation2[i]=_v2[i];
	recurrenthidden2[i]=_v2[i];
	//	   if (activation[i] > 1.0) activation[i] = 1.0;
	//	   else if (activation[i] <0.0) activation[i] =0.0;
      }
    
    // unsigned index2=0;		   // index of current weight (vector poids)
    
    
    for (k=1;k<structu2.size();k++)  //the first layer is already activated
      {
	for (i=debutCoucheNetwork2[k]; i<debutCoucheNetwork2[k+1]; i++) // neurons in current layer
	  {
	    double sss=poids[index++]; // bias
	    double sss2=sss;

	    
	    //    if (k!=structu2.size()-1)  //No recurrent connections on the output neurons
	    //  for (unsigned layinc=debutCoucheNetwork2[k]; layinc<debutCoucheNetwork2[k+1]; layinc++)
	    //	sss2 += poids[index++]*recurrenthidden2[layinc];
	    
	    
	    
	    for (j=debutCoucheNetwork2[k-1]; j< debutCoucheNetwork2[k]; j++) // from neurons of previous layer
	      {
		
		sss2 += poids[index++]*recurrenthidden2[j];
		
		

	      }
	    activation2[i]=sigmoid(sss2);
	   
	  }
      }


     

    for(unsigned  i=0;i<nbNeuroneNetwork2;i++)
      {
	
	recurrenthidden2[i]=activation2[i];
      }
    //  _sortie.resize(nbSortie);

    //  for (i=nbNeurone-nbSortie, j=0; i<nbNeurone; i++,j++)
    // {
    //	_sortie[j] = activation[i];
    // }

    /*
      if (numero ==1)
      output = _sortie;
    */
    // return _sortie;  
  }


protected:
  unsigned nbEntreeNetwork1;
  unsigned nbSortieNetwork1;
  unsigned nbEntreeNetwork2;
  unsigned nbSortieNetwork2;
  unsigned nbNeuroneNetwork1;
  unsigned nbConnectionNetwork1;
  unsigned nbNeuroneNetwork2;
  unsigned nbConnectionNetwork2;
  
  
  /////////////////////////// private data and methods /////////////////////////////
private:
  vector<double> savediff; //modif provisoire !!!!!
  vector<double> recurrenthidden2; //modif provisoire !!!!!
  vector<double> recurrenthidden; 
  vector<unsigned> lesCouchesProvi;
  vector<unsigned> lesCouches;
  vector <unsigned> lesCouches2;
  vector<unsigned> debutCoucheNetwork1;   // helper for calcul
  vector<unsigned> debutCoucheNetwork2;   // helper for calcul
  unsigned nbCoucheNetwork1;
  unsigned nbCoucheNetwork2;

  /** Mise a Jour (nbNeurones, nbSorties, ...) apres lectures de couches */
  void majCtor()
  {


    
    
    //    savearchi.resize(lesCouchesProvi.size());
    //savearchi=lesCouchesProvi;

    nbConnection =0;

    lesCouches.resize(lesCouchesProvi[0]);
    for (unsigned i=1;i<=lesCouchesProvi[0];i++)
      lesCouches[i-1]=lesCouchesProvi[i];
    nbCoucheNetwork1 = lesCouches.size();
    nbEntreeNetwork1 = lesCouches[0];
    nbSortieNetwork1 = lesCouches[nbCoucheNetwork1-1];
    debutCoucheNetwork1.resize(lesCouches.size()+1);
    // on compte les neurones et les connextions, on remplit debutCouche
    debutCoucheNetwork1[0]=0;
    nbNeuroneNetwork1 = lesCouches[0]*2; //*2 cause the delta error due to anticipation method
      nbConnectionNetwork1 = 0;
      for (unsigned i=1; i<nbCoucheNetwork1; i++)
	{
	  //	if (i==2)
	  //  debutCouche[i] = nbNeurone+lesCouches[1]-2;
	  //	else
	  debutCoucheNetwork1[i] = nbNeuroneNetwork1;

	  nbNeuroneNetwork1 += lesCouches[i];
	
	  if (i==1)
	    nbConnectionNetwork1 += lesCouches[1] *(lesCouches[0]*2+1);
	  else
	    nbConnectionNetwork1 += lesCouches[i] *(lesCouches[i-1]+1);  //we add the numbers of recurrent connections for the layer i + bias
	
	  if (i!=nbCoucheNetwork1-1) //not for the output layer
	    nbConnectionNetwork1 += lesCouches[i]*lesCouches[i];  //recurrent connection on the layer (Elman 1990)
	


	}

    
      recurrenthidden.resize(nbNeuroneNetwork1);
      for (unsigned i=0; i<recurrenthidden.size(); i++)
	{ 
	  recurrenthidden[i]=0.0;
	}
      debutCoucheNetwork1[nbCoucheNetwork1] = nbNeuroneNetwork1; // total  number of neurons == end of last layer
  
      //Network2 // For prediction


      unsigned zero2;
      zero2=nbCoucheNetwork1+1;
      unsigned nbCoucheNetwork2 = lesCouchesProvi[zero2];
     
      nbEntreeNetwork2 = lesCouches[nbCoucheNetwork1-1]+lesCouches[nbCoucheNetwork1-2];
      // for (unsigned i=0;i<lesCouchesProvi.size();i++)
     

 
      nbSortieNetwork2 = lesCouches[0];  //the number of inputs in the network 1
     
      //  vector<unsigned> test(3);
  
      //vector<unsigned> test2(3);
     

     

      lesCouches2.resize(lesCouchesProvi[zero2]);

     
   


      lesCouches2[0]=nbEntreeNetwork2;
     

      lesCouches2[nbCoucheNetwork2-1]=nbSortieNetwork2;
     
      for (unsigned i=nbCoucheNetwork1+2;i<lesCouchesProvi.size();i++)
	{
	  lesCouches2[i-nbCoucheNetwork1-1]=lesCouchesProvi[i];
	
	}

     





      debutCoucheNetwork2.resize(lesCouches2.size());

      //for (unsigned i=0;i<lesCouches2.size();i++)
     


      //  debutCoucheNetwork2.resize(3); 
     

      //exit(1);
      // on compte les neurones et les connextions, on remplit debutCouche
      debutCoucheNetwork2[0]=0;
      //nbNeuroneNetwork2 = nbEntreeNetwork1; //*2 cause the delta error due to anticipation method
      nbNeuroneNetwork2=nbEntreeNetwork2;
      nbConnectionNetwork2 = 0;
      for (unsigned i=1; i<nbCoucheNetwork2; i++)
	{
	  //	if (i==2)
	  //  debutCouche[i] = nbNeurone+lesCouches[1]-2;
	  //	else
	  debutCoucheNetwork2[i] = nbNeuroneNetwork2;

	  nbNeuroneNetwork2 += lesCouches2[i];
	  nbConnectionNetwork2 += lesCouches2[i] *(lesCouches2[i-1]+1);  //we add the numbers of recurrent connections for the layer i + bias
	
	
	  //if (i!=nbCoucheNetwork2) //not for the output layer
	  //nbConnectionNetwork2 += lesCouches2[i]*lesCouches2[i];  //recurrent connection on the layer (Elman 1990)
	
	}

     

	  nbConnection=nbConnectionNetwork1+nbConnectionNetwork2;
    

	  // exit(1);

	 	  recurrenthidden2.resize(nbNeuroneNetwork2);

	  for (unsigned i=0; i<recurrenthidden2.size(); i++)
	    { 
	      recurrenthidden2[i]=0.0;
	    }


	  debutCoucheNetwork2[nbCoucheNetwork2] = nbNeuroneNetwork2; // total  number of neurons == end of last layer

	  //----->>>>>>>>>>>>>>>>debutCoucheNetwork2[nbCoucheNetwork2] = nbNeuroneNetwork2; // total  number of neurons == end of last layer



	
    
  }



};


//_alpha for sigmoid

void backprop (vector<Real> & _desired_output, vector<Real> & _real_output, Real _learning_rate, 
	       Real _inertial_rate, Real _alpha, vector<Real> & _poids,
	       vector <Real> & _recurrenthidden2, vector<int> & _debutCouche, int _nbCouche,
	       vector <int> & _lesCouches, vector <Real> & _delta, vector<int> _debutConnection, int _max_layer);



  //Neural Network used in backprop function is coded like this way: sample sinus function:
  /**

  Sample for a network 1-5-1 , 1 input, 5 hidden neurons and 1 output

  ****         ****
  *  *         *  *                    Note : the bias is introduced on the last layer to simplify the code of network computations
  ****         ****
  bias         output to approximate sinus
  |      
  |
  _____________|________________________
  |      |      |      |      |      |
  ****   ****   ****   ****   ****   ****
  *  *   *  *   *  *   *  *   *  *   *  *                    
  ****   ****   ****   ****   ****   ****
  bias     |      |      |      |      |
  __|______|______|______|______|______|___
  |
  ________|________
  |            |  
  ****         ****  
  *  *         *  *  
  ****         ****   
  bias         input representsing x in sinus(x)


  */















  /*! 
   *  \brief Architecture of anticipation with back-propagation, simple version, the weights are modified by Artificial Evolution
   * 
   *
   *   Sample of a neural network using this archi:
   *   
   *   First part of the network:
   *
   *
   *   O----
   *   O-----                    O----
   *   O------                   O-----
   *   O-------                  O------                O
   *   O-------                  O------                O 
   *   O------                   O-----
   *   O-----                    O----
   *   O----
   * First Layer (L1)           Second Layer (L2)           Last Layer (L3)
   * (sensors activitites)   (hidden Layer)       (Output neurons - typically the motors activations)
   *
   *   Fully network
   *
   *   
   *   L1 ------->  L2  -----------> L3
   *          >      |                |    
   *          |      |                |
   *          |      ------------------
   *          |              ||
   *          |             \ /
   *   O      |           O O O O O   (Hidden layer of the second network) (L4)
   *   O      |              |
   *   O      |              |
   *   O -----|              |
   *   O                    \/ 
   *   O (L6)       <--  O O O O O O O O (Prediction of the sensor activation at t+1) (L5)        
   *   O      (we put the deltas between the prediction of the activtion at t+1 and the real senations obtained at t+1) 
   *   O     
   * Let's note that this network is ELMAN type, i.e. the hidden layer is fully recurrent 
   *
   *
   *  So the fully network is in fact :
   *
   *   L1 ----/
   *          / ---  L2  ------> L3
   *   L6 ----/       |           |
   *                  |           |
   *                  -------------
   *                        |  
   *                       L4
   *                        |
   *                       L5 
   *
   *  \author Nicolas Godzik
   *  \version 1.0
   *  \date    December 2003
   *  \bug No
   *  \warning 
   */



  class AnticipationNetworkBackprop : public eoArchi //for recurrency in 
  {
  public:


    /** Ctor from parameters (e.g. as read from the parser) */
    AnticipationNetworkBackprop (vector<unsigned> & _lesC): lesCouchesProvi(_lesC)//, _debutCouche(_lesC.size()+1)
    {
      majCtor();
    }
  
    /** Ctor from an istream - uses readFrom */
    AnticipationNetworkBackprop(istream & _is)
    {
      readFrom(_is);
    }

    virtual ~AnticipationNetworkBackprop(){}
  
    virtual string className() {return "AnticipationNetworkBackprop";}

    virtual void printOn(ostream & _os)
    {
      _os << className() << " " ;
      _os <<  lesCouchesProvi.size() << " " ;
      for (unsigned i=0; i<lesCouchesProvi.size(); i++)
	_os <<  lesCouchesProvi[i] << " ";
   
    }
  
    virtual void readFrom(istream & _is)
    {
      unsigned nbCoucheProvi;
      _is >> nbCoucheProvi;
      lesCouchesProvi.resize(nbCoucheProvi);
      //debutCoucheNetwork1.resize(nbCouche+1);

      for (unsigned i=0; i<nbCoucheProvi; i++)
	_is >> lesCouchesProvi[i];
      majCtor();
    }
  
    virtual void erase_recurrent ()
    {
      //For each epoch execute this if erase_each_epoch is activated


      
      if (erase_each_epoch)
	{
	  for (unsigned i=0; i<recurrenthidden.size(); i++)
	    { 
	      recurrenthidden[i]=0.0;
	    }
	  for (unsigned i=0; i<recurrenthidden2.size(); i++)
	    { 
	      recurrenthidden2[i]=0.0;
	    }

	}

      //Execute this in any case
      //Put Delta to zero (for inertial term) at the beginning
      
      for (unsigned i=0; i<delta.size(); i++)
	 { 
	      delta[i]=0.0;
	    }


    }

    virtual void calcul(vector<Real> & poids, vector<Real> & _v, 
			vector<Real> & _sortie)
    {

      
      
      int i,j,k;
            vector<int> & structu = lesCouches;
      vector<Real> activation(nbNeuroneNetwork1);
      recurrenthidden.resize(nbNeuroneNetwork1);
      recurrenthidden2.resize(nbNeuroneNetwork2);

            input_network1=_v;

      _v.resize(lesCouches[0]*2);
    
      savediff.resize(lesCouches[0]);


      for (int  i=0;i<structu[0];i++)
	{

	  _v[structu[0]+i]=fabs(memory_output2[i]-_v[i]);  //Add delta to the input of the network1
	  savediff[i]=_v[structu[0]+i];
	  //	   if (activation[i] > 1.0) activation[i] = 1.0;
	  //	   else if (activation[i] <0.0) activation[i] =0.0;
	}


      //Copy necessary vector for desired output (input in fact)


      //  start = _v.begin();
      //  end = _v.begin()+lesCouches[0];

      //copy(start,end,input_network1.begin());



      //The last two poids are in fact learning_rate and inertial_rate

      //      start = poids.end()-2;

      
      //      if (!first) 
      //	{
vector<Real>::iterator start,end;
      start = poids.begin()+nbConnectionNetwork1;
      
      end = poids.end()-2;
      
      copy(start,end,weights_network_2.begin());
      

    
      backprop(input_network1,memory_output2,fabs(*(end+1)),fabs(*(end+2)),alpha,weights_network_2,recurrenthidden2,debutCoucheNetwork2,nbCoucheNetwork2,lesCouches2,delta,debutConnectionNetwork2,max_layer);

           // copy of input layer
      for (int  i=0;i<structu[0]*2;i++)
	{
	  activation[i]=_v[i];
	  recurrenthidden[i]=_v[i];
	}
    
     
      int index=0;		   // index of current weight (vector poids)
      for (k=1;k<int(structu.size());k++)  //the first layer is already activated
	{
	  for (i=debutCoucheNetwork1[k]; i<debutCoucheNetwork1[k+1]; i++) // neurons in current layer
	    {
	      Real sss=poids[index++]; // bias
	      Real sss2=sss;

	    
	      for (j=debutCoucheNetwork1[k-1]; j< debutCoucheNetwork1[k]; j++) // from neurons of previous layer
		{
		
		  sss2 += poids[index++]*recurrenthidden[j];
		
		}
	    
	      if (k!=int(structu.size())-1)  //No recurrent connections on the output neurons
		for (int layinc=debutCoucheNetwork1[k]; layinc<debutCoucheNetwork1[k+1]; layinc++)
		  sss2 += poids[index++]*recurrenthidden[layinc];


	    	     activation[i]=sigmoid(sss2); //For the last layer, classical computation

		     //	      if (k==int(structu.size())-1) activation[i]=sigmoid(sss2); //For the last layer, classical computation
		     //else activation[i]=trigger (sss2,activation[i]);//A little memory like in the hidden neurons
	    }
	}
   

      
     

      for(int  i=0;i<nbNeuroneNetwork1;i++)
	{
	 
	  recurrenthidden[i]=activation[i];
	 
	}
      
      _sortie.resize(nbSortieNetwork1);
      
      //And now for the network2
      //We copy the weights dedicated to the second network

      //Now We have our weights vector for the second network
    
      for (i=nbNeuroneNetwork1-nbSortieNetwork1, j=0; i<nbNeuroneNetwork1; i++,j++)
	{
	 
	  _sortie[j] = activation[i];
	 
	}

     
      
      vector<int> & structu2 = lesCouches2;
      vector<Real> activation2(nbNeuroneNetwork2);
      recurrenthidden2.resize(nbNeuroneNetwork2);

      //We fill the input vector _v2 which represents the input entrance for the second network i.e. in the first network, we take all hiddens neurons
      //in ELMAN network and we take all outputs

      vector<Real> _v2;

      _v2.resize(structu2[0]);

         for (int i=0;i<int(lesCouchesProvi[nbCoucheNetwork1-1]);i++)
	{
	_v2[i]=recurrenthidden[nbNeuroneNetwork1-lesCouchesProvi[nbCoucheNetwork1]-lesCouchesProvi[nbCoucheNetwork1-1]+i];
	
	}
	
      for (int i=0;i<int(lesCouchesProvi[nbCoucheNetwork1]);i++)
	{
	_v2[lesCouchesProvi[nbCoucheNetwork1-1]+i]=recurrenthidden[nbNeuroneNetwork1-lesCouchesProvi[nbCoucheNetwork1]+i];
	
	}

      vector<Real> _sortie2(nbSortieNetwork2);

      calcul_anticipation_network(weights_network_2, _v2, _sortie2);

          
      for(int  i=0;i<nbSortieNetwork2;i++)
	{
	 
	  memory_output2[i]=_sortie2[i];
	 
	}

         
    }


    //The network implement here is optimised for the computation of backpropagation after, so it differs a little from calcul function, the bias neuron is completely integrated in each layer, to make it easier after to backpropagate the error

    void calcul_anticipation_network(vector<Real> & _poids, vector<Real> & _v, 
				     vector<Real> & _sortie_network2)
    {
      

        int j,k;
       vector<int> & structu2 = lesCouches2;
       Real output;
     
      _sortie_network2.resize(nbSortieNetwork2);
       recurrenthidden2[0]=1.0; //bias
      for (int  i=1;i<structu2[0];i++)
      	{
      	  recurrenthidden2[i]=_v[i-1];
       
      	}
      int index=0;		   // index of current weight (vector poids)
      int debutsortie = 0;
     
      for (k=1;k<int(structu2.size());k++)  //the first layer is already activated
      	{
        for (int i=debutCoucheNetwork2[k]; i<debutCoucheNetwork2[k+1]; i++) // neurons in current layer
          {
	          Real sum=0.0;//_poids[index++]; // bias
	      recurrenthidden2[debutCoucheNetwork2[k]]=1.0; //bias //if (k!=nbCouche-1)
            if (i!=debutCoucheNetwork2[k])
          
      	{
      	  for (j=debutCoucheNetwork2[k-1]; j< debutCoucheNetwork2[k]; j++) // from neurons of previous layer
      	    {
	       sum += _poids[index++]*recurrenthidden2[j];
      
      	    }
      
      	}
	
	     output=sigmoid_alpha(sum,alpha);
	     if (i!=debutCoucheNetwork2[k]) recurrenthidden2[i]=output;
	     if ((k==(nbCoucheNetwork2-1)) && (i!=debutCoucheNetwork2[k])) {recurrenthidden2[i]=output;_sortie_network2[++debutsortie]=recurrenthidden2[i];}
	     else if ((k==(nbCoucheNetwork2-1)) && (i==debutCoucheNetwork2[k])) {_sortie_network2[0]=1.0;}

	  }
      	}

    }



  protected:
    int nbEntreeNetwork1;
    int nbSortieNetwork1;
    int nbEntreeNetwork2;
    int nbSortieNetwork2;
    int nbNeuroneNetwork1;
    int nbConnectionNetwork1;
    int nbNeuroneNetwork2;
    int nbConnectionNetwork2;
    int max_layer;
  
  
    /////////////////////////// private data and methods /////////////////////////////
  private:
    vector<Real> savediff; //modif provisoire !!!!!
    vector<Real> recurrenthidden2; //modif provisoire !!!!!
    vector<Real> recurrenthidden; 
    vector<unsigned> lesCouchesProvi;
    vector<int> lesCouches;
    vector<int> lesCouches2;
    vector<int> debutCoucheNetwork1;   // helper for calcul
    vector<int> debutCoucheNetwork2;   // helper for calcul
    vector<int> debutConnectionNetwork2;
    vector<Real> memory_output2;   // helper for calcul
    vector<Real> input_network1;
    vector<Real> delta;

    vector<Real> weights_network_2;
    vector<Real>::iterator start,end;
  
    int nbCoucheNetwork1;
    int nbCoucheNetwork2;
    Real alpha; //for network activity using alpha.x instead of x (sigmoid_alpha)

    /** Mise a Jour (nbNeurones, nbSorties, ...) apres lectures de couches */
    void majCtor()
    {
      nbConnection =0;

      

      lesCouches.resize(lesCouchesProvi[0]);
      for (int i=1;i<=int(lesCouchesProvi[0]);i++)
	lesCouches[i-1]=lesCouchesProvi[i];


      
     

      nbCoucheNetwork1 = lesCouches.size();
      nbEntreeNetwork1 = lesCouches[0];
      nbSortieNetwork1 = lesCouches[nbCoucheNetwork1-1];
      debutCoucheNetwork1.resize(lesCouches.size()+1);


 
           // on compte les neurones et les connextions, on remplit debutCouche
      debutCoucheNetwork1[0]=0;
      nbNeuroneNetwork1 = lesCouches[0]*2; //*2 cause the delta error due to anticipation method
	nbConnectionNetwork1 = 0;

 
	for (int i=1; i<nbCoucheNetwork1; i++)
	  {
	    debutCoucheNetwork1[i] = nbNeuroneNetwork1;
 
	    nbNeuroneNetwork1 += lesCouches[i];
	
	    if (i==1)
	      nbConnectionNetwork1 += lesCouches[1] *(lesCouches[0]*2+1);
	    else
	      nbConnectionNetwork1 += lesCouches[i] *(lesCouches[i-1]+1);  //we add the numbers of recurrent connections for the layer i + bias
	
	    if (i!=nbCoucheNetwork1-1) //not for the output layer
	      nbConnectionNetwork1 += lesCouches[i]*lesCouches[i];  //recurrent connection on the layer (Elman 1990)
	


	  }
   
		recurrenthidden.resize(nbNeuroneNetwork1);
	for (int i=0; i<int(recurrenthidden.size()); i++)
	  { 
	    recurrenthidden[i]=0.0;
 
	  }
	debutCoucheNetwork1[nbCoucheNetwork1] = nbNeuroneNetwork1; // total  number of neurons == end of last layer
 
	
	int zero2;
	zero2=nbCoucheNetwork1+1;
	nbCoucheNetwork2 = lesCouchesProvi[zero2];
 
	nbEntreeNetwork2 = lesCouches[nbCoucheNetwork1-1]+lesCouches[nbCoucheNetwork1-2];
	nbSortieNetwork2 = lesCouches[0]+1;  //the number of inputs in the network 1
	
	lesCouches2.resize(lesCouchesProvi[zero2]);
	lesCouches2[0]=nbEntreeNetwork2;
	lesCouches2[nbCoucheNetwork2-1]=nbSortieNetwork2;
 
	for (int i=nbCoucheNetwork1+2;i<int(lesCouchesProvi.size());i++)
	  {
	    lesCouches2[i-nbCoucheNetwork1-1]=lesCouchesProvi[i];
	  }

		for (int i=0;i<(int)lesCouches2.size()-1;i++)
	  lesCouches2[i]++;  //because of bias - The Network sumitted to back propagation is coded like this way (one neuron more per layer to make backpropagation computation easier in the dedicated function

	//      size_weights_back_prop.resize(lesCouches2[1]*(lesCouches2[0]+1)+lesCouches2[2]*(lesCouches2[1]+1));
  
	
	//exit (-1);

	//  nbCouche = lesCouches.size();

	//  nbEntree = lesCouches[0];
	//  nbSortie = lesCouches[nbCouche-1];


	// on compte les neurones et les connexions, on remplit debutCouche
	debutCoucheNetwork2.resize(lesCouches2.size()+1);
	debutConnectionNetwork2.resize(lesCouches2.size());
	debutCoucheNetwork2[0]=0;

	
	//We gonna count the bias also in the debutCouche so +1
  
	nbNeuroneNetwork2=nbEntreeNetwork2+1;
	nbConnectionNetwork2 = 0;
	debutConnectionNetwork2[0]=0;
	max_layer=0;
	
	
	for (int i=1; i<nbCoucheNetwork2; i++)
	  {
	    debutCoucheNetwork2[i] = nbNeuroneNetwork2;
	    nbNeuroneNetwork2 += lesCouches2[i];
	    if (lesCouches2[i]>max_layer) max_layer=lesCouches2[i];
	    nbConnectionNetwork2 += (lesCouches2[i]-1) *(lesCouches2[i-1]); 
	    debutConnectionNetwork2[i]+=nbConnectionNetwork2;
	  }

	debutCoucheNetwork2[nbCoucheNetwork2]=nbNeuroneNetwork2;

	nbConnection=nbConnectionNetwork1+nbConnectionNetwork2;

	recurrenthidden2.resize(nbNeuroneNetwork2);
	weights_network_2.resize(nbConnectionNetwork2);
	for (int i=0; i<int(recurrenthidden2.size()); i++)
	  { 
	    recurrenthidden2[i]=0.0;
	  }

	
	delta.resize(nbConnectionNetwork2);
	
	debutCoucheNetwork2[nbCoucheNetwork2] = nbNeuroneNetwork2; // total  number of neurons == end of last layer
	
	memory_output2.resize(nbSortieNetwork2);
	alpha=4.0; //For sigmoid
	
  
    }



  };


#endif
