/** -*- mode: c++; c-indent-level: 4; c++-member-init-indent: 8; comment-column: 35; -*-

The above line is usefulin Emacs-like editors
*/

/*
  Template for EO objects initialization in EO
  ============================================
*/

#ifndef _eoRNInit_h
#define _eoRNInit_h

#ifndef DONT_USE_EO

// include the base definition of eoInit
#include <eoInit.h>
#include <sstream>

#include "map_controllers.h"
#include "global.h"
#include "list_controllers.h"
#include "robot.h"
#include "world.h"

extern world world_GL;

//#include <utils/eoRndGenerators.h>

/** 
 *  Initialisation of eoRNGenotype
 *  In the constructor of this class, we will read in the parser all the necessary parameters for the genotype that is to say the weights of several robots
 *  The functor () will enable to all the populations to have a pointer to its architecture first of all, and of course initialize the weights of the archi for example
 */
class eoRNGenotypeInit: public eoInit<eoRNGenotype> {
public:
  /** Ctor - read permanent parameters */
  eoRNGenotypeInit(eoParser & _parser, world & _world_eo) 
  {
    read_archi(_parser, _world_eo);

    // now some initial value for sigmas - even if useless?
    // should be used in Normal mutation
    // meme sigmaInit pour tous (sera normalise par le range)
    sigmaInit = _parser.createParam(double(0.3), "sigmaInit", "Initial value for Sigma(s)", 's', "RN - Genotype Initialization").value(); //+_num

    // minimum check
    if ( (sigmaInit < 0) )
      throw runtime_error("Invalid sigma in eoRNInit Ctor");

  }

  // we have constructed a pointer in the Ctor, we must delete it
  virtual ~eoRNGenotypeInit() {
    for (unsigned i=0; i<archil.size();i++)
    delete archil[i];
  }

  /** initializes a RN */
  void operator()(eoRNGenotype & _rn)
  {
    

    _rn.number_of_robots=nrobots;
   
    (_rn.robots).resize(nrobots);
 
    for (unsigned i=0;i<_rn.number_of_robots;i++)
      {
	_rn.robots[i].archi = archil[i];
	//Initialisation du genotype:
	unsigned nbConnection = archil[i]->NbPoids();
	_rn.robots[i].poids.resize(nbConnection);
    
	_rn.robots[i].stDev.resize(nbConnection);
	for (unsigned j=0;j<nbConnection;j++)
	  {
	    _rn.robots[i].poids[j] = bounds->uniform(j);
	    _rn.robots[i].stDev[j] = sigmaInit * bounds->range(j);
	  }
	_rn.robots[i].controller=controller_base[i];
	_rn.robots[i].string_controller=save_strings[i];
	
      }

  }

  ///////////// PRIVATE DATA //////////////////////////

private:
  vector<eoArchi  *> archil;
  eoRealVectorBounds * bounds;
  double sigmaInit;
  vector<Controller *> controller_base;
  vector<string> save_strings; //It will be very useful after when we will save the strings of the function controllers used during the evolution
  unsigned nrobots; //number of robots
  vector<string> type_of_robots;
  vector<robot *> list_of_robots;
  vector<position_robot> vector_pos_robot;

  /** private function read_archi */
  void read_archi(eoParser & _parser, world & _world_eo)
  {


    nrobots = _parser.createParam(unsigned(1), "robots", "Number of Robots", '\0', "Genotype Initialization").value();
	
    archil.resize(nrobots);
    //  bounds.resize(nrobots);
    // sigmaInit.resize(nrobots);
    controller_base.resize(nrobots);
    // save_strings=new vector<string>;
    save_strings.resize(nrobots);
    type_of_robots.resize(nrobots);
    list_of_robots.resize(nrobots);

    for (unsigned i=0; i < nrobots; i++)
      { 
	//Real obstacles[5];

	ostringstream paramErase; // ostrstream os(paramName);blackbox blackbox;
	paramErase << "erase_each_epoch" << i ; //os << "connexion" << i << ends;
	bool erase_each_epoch  = _parser.createParam(unsigned(false), paramErase.str(), "If activated, the activations of the hidden neurons will be erased on each epoch, it works only with the recurrent networks ! like Elman", '\0', "Genotype Initialization").value();
	
	ostringstream paramName; // ostrstream os(paramName);blackbox blackbox;

	paramName << "archi" << i ; //os << "connexion" << i << ends;
	eoParamParamType & ptArchi = _parser.createParam( eoParamParamType("MULTILAYER(2,8,2)"),paramName.str(), "Architecture de la boite noire", '\0', "Genotype Initialization").value();
	   
	if (ptArchi.first == "MULTILAYER")
	  {
	    vector<string> & vString = ptArchi.second;
	    if (!vString.size())
	      throw runtime_error("Invalid data in read_archi MULTILAYER");
	    unsigned NbCouche = atoi(vString[0].c_str());
	    if (vString.size() != NbCouche+1)
	      throw runtime_error("Invalid number of layers in read_archi MULTILAYER");
	    vector<unsigned> lesCouchesp(NbCouche);
	    for (unsigned j=0; j<NbCouche; j++)
	      lesCouchesp[j] = atoi(vString[j+1].c_str());
	    archil[i] = new eoMultiLayer(lesCouchesp);
	  }

	

	// feedforward: 
	else if (ptArchi.first == "FEEDFORWARD")
	  {
	    
	    vector<string> & test = ptArchi.second;

	    unsigned nbEntree = atoi(test[0].c_str());
	    unsigned nbCachee = atoi(test[1].c_str()) ;
	    unsigned nbSortie = atoi(test[2].c_str()) ;
	    unsigned nbConn = atoi(test[3].c_str()) ;


	    unsigned nbN = nbCachee+nbEntree+nbSortie;
	    vector<bool> connectivite(nbN*nbN, false);

	    for (unsigned j=0; j<nbConn; j++)
	      {
		// string paramName;
		ostringstream paramName; // ostrstream os(paramName);
		paramName << "connection" << j ; 
		eoParamParamType & ptArchi2 = _parser.createParam( 
								  eoParamParamType("(0,0)"),
								  paramName.str(), 
								  "Definition feed forward", '\0', 
								  "RN - Genotype Initialization").value(); 
	
		vector<string> & conns = ptArchi2.second;
		if (conns.size() != 2)
		  throw runtime_error("Invalid nb of connections");
		unsigned i0 = atoi(conns[0].c_str());
		unsigned i1 = atoi(conns[1].c_str());
		connectivite[i0+i1*nbN]=true;
	      }
	    eoFeedForward * architmp = new eoFeedForward(nbEntree, nbCachee, nbSortie, connectivite);
	    archil[i] = architmp;
	  }
	// recurrent: 
	else if (ptArchi.first == "ELMAN")  //For the problem of the exploring environment and find the light
	  {
	

	    vector<string> & vString = ptArchi.second;

	    if (!vString.size())
	      throw runtime_error("Invalid data in read_archi MULTILAYER");
	    unsigned NbCouche = atoi(vString[0].c_str());
	
	    vector<unsigned> lesCouchesp(NbCouche);
	    for (unsigned j=0; j<NbCouche; j++)
	      lesCouchesp[j] = atoi(vString[j+1].c_str());
	    archil[i] = new eoRecurrentHidden(lesCouchesp);
	    archil[i]->erase_each_epoch = erase_each_epoch;
	  }

	// recurrent && memory: 
	else if (ptArchi.first == "RECURRENTHIDDENTRIGGER")  //For the problem of the exploring environment and find the light
	  {
	

	    vector<string> & vString = ptArchi.second;

	    if (!vString.size())
	      throw runtime_error("Invalid data in read_archi RECURRENTHIDDENTRIGGER");
	    unsigned NbCouche = atoi(vString[0].c_str());
	    if (vString.size() < 5)
	      throw runtime_error("Invalid number of layers in read_archi RECURRENTHIDDENTRIGGER MUST BE AT LEAST 3 BECAUSE OF THE RECURRENT HIDDEN LAYERS AND MEMORY");
	    vector<unsigned> lesCouchesp(NbCouche);
	    for (unsigned j=0; j<NbCouche; j++)
	      lesCouchesp[j] = atoi(vString[j+1].c_str());
	    archil[i] = new eoRecurrentHiddenTrigger(lesCouchesp);
	    archil[i]->erase_each_epoch = erase_each_epoch;
	  }

	else if (ptArchi.first == "ANTICIPATIONNETWORK")  //For the problem of the exploring environment and find the light
	  {
	
	    vector<string> & vString = ptArchi.second;
	    if (!vString.size())
	      throw runtime_error("Invalid data in read_archi ANTICIPATIONNETWORK");
	    unsigned NbCouche1 = atoi(vString[0].c_str());
	    unsigned NbCouche2 = atoi(vString[NbCouche1+1].c_str());
	    unsigned NbCouche = NbCouche1+NbCouche2;
	    if (vString.size() != NbCouche)
	      throw runtime_error("Invalid number of layers in read archi ANTICIPATIONNETWORK (ex: 3,8,5,2,3,5)");
	    vector<unsigned> lesCouchesp(NbCouche);
	    for (unsigned j=0; j<NbCouche; j++)
	      lesCouchesp[j] = atoi(vString[j].c_str());  //vstring [j+1]
	    archil[i] = new AnticipationNetwork(lesCouchesp);
	    archil[i]->erase_each_epoch = erase_each_epoch;
	  }
	
	else if (ptArchi.first == "ANTICIPATIONNETWORKBACKPROP")  //For the problem of the exploring environment and find the light
	  {
	    vector<string> & vString = ptArchi.second;
	    if (!vString.size())
	      throw runtime_error("Invalid data in read_archi ANTICIPATIONNETWORKBACKPROP");
	    unsigned NbCouche1 = atoi(vString[0].c_str());
	    unsigned NbCouche2 = atoi(vString[NbCouche1+1].c_str());
	    unsigned NbCouche = NbCouche1+NbCouche2;
	    if (vString.size() != NbCouche)
	      throw runtime_error("Invalid number of layers in read archi ANTICIPATIONNETWORKBACKPROP (ex: 3,8,5,2,3,5)");
	    vector<unsigned> lesCouchesp(NbCouche);
	    for (unsigned j=0; j<NbCouche; j++)
	      lesCouchesp[j] = atoi(vString[j].c_str());  //vstring [j+1]
	    archil[i] = new AnticipationNetworkBackprop(lesCouchesp);
	    archil[i]->erase_each_epoch = erase_each_epoch;
	  }
	

	else throw runtime_error("Only MULTILAYER and FF defined at the moment and ELMAN and RECURRENTHIDDENTRIGGER and ANTICIPATION NETWORK ANTICIPATIONNETWORKBACKPROP"); 


	// les bornes (comme sigmaInit) sont les memes pour tous pour l'instant
	unsigned nbP = archil[i]->NbPoids();
	cout << "Number of weights " << nbP << endl;
	bounds = &_parser.createParam(eoRealVectorBounds(nbP,-1,1), 
				      "initBounds", 
				      "Bounds for initialization (MUST be bounded)", 
				      'B', 
				      "RN - Genotype Initialization").value();
      

	ostringstream paramStringName; // ostrstream os(paramName);
	paramStringName << "controller_functor" << i ; // Name of the function controller which is in definitive a functor
	string & ptStringArchi = _parser.createParam( string ("controller_with_blackbox"),paramStringName.str(), "class controller", '\0', "Genotype Initialization").value();
	   
	save_strings[i]=ptStringArchi;

#ifdef _WIN32
    Controller * mc =(static_cast<Controller *> (all_the_controllers(save_strings[i])));
#else
    Controller * mc =(dynamic_cast<Controller *>(all_the_controllers(save_strings[i])));
#endif
	
	controller_base[i]=mc;
	
	//	controller_base[i]->test();
	//	exit (1);

	
	 unsigned  npos = _parser.createParam(unsigned(0), "npos", "Number of positions, the robot will change its position every new epoch", '\0', "Genotype Initialization").value();
	 vector_pos_robot.resize(npos);

	 for (unsigned j=0; j < npos; j++)
	   { 
	     position_robot  position_provi;
	    
	     ostringstream paramName; // ostrstream os(paramName);
	     paramName << "pos" << i<< "_" <<j ; //os << "connexion" << i << ends;
	     eoValueParam<eoParamParamType>& connParam = _parser.createParam( eoParamParamType("(0.0,0.0,M_PI)"),paramName.str(), "Definition for Position", '\0', "Genotype Initialization"); //paramName.s_str()
	     eoParamParamType & ptArchi2 = connParam.value();
	     vector<string> & conns = ptArchi2.second;
	     if (conns.size() != 3)
	       throw runtime_error("Invalid nb of parameters for position, haven't you forgotten the direction in radians at the end?");
	     position_provi.centerx = atof(conns[0].c_str());
	     position_provi.centery =  atof(conns[1].c_str());
	     position_provi.direction = atof(conns[2].c_str());
	     	    
	     vector_pos_robot[j]=position_provi;	    
	   }
	 
	 
	 ostringstream paramStringType; // ostrstream os(paramName);
	 paramStringType << "type_of_robot" << i ; // Name of the function controller which is in definitive a functor
	 string & ptStringType = _parser.createParam( string ("khepera_EO"),paramStringType.str(), "Type de robot", '\0', "Genotype Initialization").value();
	 type_of_robots[i]=ptStringType;
	 
	 if (npos==0)
	   {

	 if (type_of_robots[i] == "khepera_EO") list_of_robots[i]=new khepera_EO (paramStringType.str(),archil[i], controller_base[i]);//,save_strings[i]); // new constructor
	 //else if ...
	 else throw breakerror ("Your type of robot doesn't exist !");
	 
	   }

	 else if (npos !=0)
	   {

	     if (type_of_robots[i] == "khepera_EO") list_of_robots[i]=new khepera_EO (paramStringType.str(),archil[i], controller_base[i],vector_pos_robot);//,save_strings[i]); // new constructor

	   	     else throw breakerror ("Your type of robot doesn't exist !");

	     cout << "COnstructor fine" <<endl;
	   }
	 
	 _world_eo.add_robot(*list_of_robots[i]);
	 world_GL.add_robot(*list_of_robots[i]);
	 
      }

  }

};

/** 
 * Now the init of the EO objects
 *
 * Needs to be templatized because of the fitness
 */
template <class GenotypeT>
class eoRNInit : public eoInit<GenotypeT>
{
public:
  eoRNInit(eoParser & _parser, world & _world) : init(_parser, _world)
  {}

  /** initializes a RN */
  void operator()(GenotypeT & _rn)
  {
    init(_rn);
    _rn.invalidate();
  }

private:
  eoRNGenotypeInit init;
};

#endif
#endif
