/** -*- mode: c++; c-indent-level: 4; c++-member-init-indent: 8; comment-column: 35; -*-

The above line is useful in Emacs-like editors
 */

/*
Template for simple mutation operators
======================================
*/

#ifndef eoRNMutation_H
#define eoRNMutation_H

#ifndef DONT_USE_EO
#include <eoOp.h>

/** 
 * Adaptive mutation for RN genotype
 */
//double stdev_eps = 1e-40;

template <class GenotypeT>
class eoRNMutation: public eoMonOp<GenotypeT>
{
public:
  /**
   * Ctor - no requirement
   */
  eoRNMutation(eoParser & _parser, string _num=""):firstTime(true),
     bounds(_parser.createParam(eoRealVectorBounds(1,-1,1), 
                    "operatorBounds" +_num, 
		    "Bounds for operators", 'b', 
		    "RN" + _num + " Operateurs").value()
           ),
    TauGlb (_parser.createParam(1.0, "TauGlb" +_num, 
             "Global Tau (before normalization)", '\0', 
	     "RN" + _num + " Operateurs").value() 
	     ),
    TauLcl (_parser.createParam(1.0, "TauLcl" +_num, 
             "Local Tau (before normalization)", '\0', 
	     "RN" + _num + " Operateurs").value() 
            ),
    cName("eoRNMutation"+_num)
  {}

  /// The class name. Used to display statistics
  string className() const { return cName; }

  /**
   * modifies the parent
   * @param _genotype The parent genotype (will be modified)
   */
  bool operator()(GenotypeT & _eo) 
  {


    for (unsigned i=0;i<_eo.number_of_robots;i++)
      {

    if (firstTime)
      {
	//	mise a lechelle des Taux;
	int taille = _eo.robots[i].archi->NbPoids();
	TauGlb = TauGlb * pow((2*taille),-0.5); 
TauLcl = TauLcl * pow((2*pow(taille,0.5)),-0.5);
	firstTime = false;
      }

    // START code for mutation of the _genotype object
     double global = TauGlb * rng.normal();
     for (unsigned j = 0; j < _eo.robots[i].archi->NbPoids(); j++)
       {
	 double stdev_eps = 1e-40;
double stdev = _eo.robots[i].stDev[j];
stdev *= exp( global + TauLcl * rng.normal() );
	
	 if (stdev < stdev_eps)
	   stdev = stdev_eps;
	 _eo.robots[i].stDev[j] = stdev;
	 
	 _eo.robots[i].poids[j] += stdev * rng.normal();
       
}
     bounds.foldsInBounds(_eo.robots[i].poids);
      
}
    
     return true;
  }

private:
  bool firstTime;
  eoRealVectorBounds & bounds;
  double TauGlb;
  double TauLcl;
  string cName;			   // className, because of possible numerotation
};

#endif
#endif
