 /*
 *Test Open GL simulator, two windows, one for the move of the robot and one for the sensors ...
 *
 */

//Classics headers

//#include <stdlib.h>
//#include <math.h>
//#include <stdio.h>
//#include <iostream>


#include "global.h"
#include <stdio.h>
#include <iostream>

#ifndef DONT_USE_EO

#include "Real.h"

Real compute_fitness(class world * w);

#include <eo>
#include "eoRN.h"
#include "eoArchi.h"
#include "param_eo.h"

typedef Real MyFitT ;	// type of fitness // In this experiment of robotics, we want to maximize the fitness ... so => double

// Then define your EO objects using that fitness type
typedef eoRN<MyFitT> Indi;      // ***MUST*** derive from EO



/** definition of representation: 
 * class eoRN MUST derive from EO<FitT> for some initial fitness
 */

// the real bounds (not yet in general eo include)
#include "utils/eoRealVectorBounds.h"



//Controller

#include "map_controllers.h"
#include "blackbox_EO.h"
#include "controller_sample.h"


//Environment, Robot,  Word and Controllers headers
#include "../../src/heart/simulator.h"


/** definition of initilization: 
 * class eoRNInit MUST derive from eoInit<eoRN>
 */
#include "eoRNInit.h"

#include <do/make_run.h>


//"global" variables used for eo 
//extern typedef Indi; // I don't if it will work or not !
eoPop <Indi> * pop;
//eoContinue<Indi> * term;
eoCheckPoint<Indi> * checkpoint;
eoAlgo<Indi> *ga;

bool eo_activated=false;
bool draw_eo=false;


void run_ea(eoAlgo<Indi>& _ga, eoPop<Indi>& _pop)
{
  do_run(_ga, _pop);
}






// To use time to initialise Random number generator
#include <time.h>

//#include <setjmp.h> // for setjmp and longjmp 
//extern jmp_buf jumper; // for an image of the stack frame entries 

// Windows parameters

#define WIDTH  500  //just for test 
#define HEIGHT 500  //just for test

/*
#define TITLE  "Test Khepera"
#define TITLE2 "Sensors Display"
*/

//vector<dsp*> vector_to_display;

//prototype of funtions and variables used after the main , we can put it in a .h too


//extern int winIdMain;
//extern int winIdSensors;
//extern int winIdControl;

extern world world_GL;  // this is the world used by the main function and the main Display function for openGL

//includes for eo




// crceate an initializer
#include "make_genotype_RN.h"
eoRNInit<Indi> & make_genotype(eoParser& _parser, eoState& _state, world& _world)
{
  return do_make_genotype(_parser, _state, _world);
} 

// and the variation operaotrs
#include "make_op_RN.h"
eoGenOp<Indi>&  make_op(eoParser& _parser, eoState& _state, eoInit<Indi>& _init)
{
  return do_make_op(_parser, _state, _init);
}

// Use existing modules to define representation independent routines
// These are parser-based definitions of objects

// how to initialize the population 
// it IS representation independent if an eoInit is given
#include <do/make_pop.h>
eoPop<Indi >&  make_pop(eoParser& _parser, eoState& _state, eoInit<Indi> & _init)
{
  return do_make_pop(_parser, _state, _init);
}

// the stopping criterion
#include <do/make_continue.h>
eoContinue<Indi>& make_continue(eoParser& _parser, eoState& _state, eoEvalFuncCounter<Indi> & _eval)
{
  return do_make_continue(_parser, _state, _eval);
}

// outputs (stats, population dumps, ...)
#include <do/make_checkpoint.h>
eoCheckPoint<Indi>& make_checkpoint(eoParser& _parser, eoState& _state, eoEvalFuncCounter<Indi>& _eval, eoContinue<Indi>& _continue) 
{
  return do_make_checkpoint(_parser, _state, _eval, _continue);
}

// evolution engine (selection and replacement)
#include <do/make_algo_scalar.h>
eoAlgo<Indi>&  make_algo_scalar(eoParser& _parser, eoState& _state, eoEvalFunc<Indi>& _eval, eoContinue<Indi>& _continue, eoGenOp<Indi>& _op)
{
  return do_make_algo_scalar(_parser, _state, _eval, _continue, _op);
}


// the instanciating fitnesses
#include <eoScalarFitness.h>


// checks for help demand, and writes the status file
// and make_help; in libutils
void make_help(eoParser & _parser);

//end of eo includes

//parameters 

#include "map_fitness.h"

void check_param_environment (eoParser& _parser,environment & _env);
void check_param_problem (map_fitnesses & _all_fitness_available, world & _world_eo, eoParser & _parser);




static int robot_display = 0; //Default individual to display is the best i.e. 0
bool new_epoch=false;  //If true, the epoch is just new (cycle=0)





/**
 *
 * look_archi()
 *
 * In this function, we look when desired (for example at the beginning of each epoch) if the architecture (ex. ELMAN) needs
 * initialised.
 *
 *
 */

void look_archi(world * w)
{
  for (vector<robot*>::iterator it1=w->vector_robot.begin();it1!=w->vector_robot.end();it1++) 
	{
	  if ((*it1)->type()=="khepera_EO") {
		
#ifdef _WIN32
		khepera_EO * robot_provi =(static_cast<khepera_EO *>( *it1));
#else
		khepera_EO * robot_provi =(dynamic_cast<khepera_EO *>( *it1));
#endif
			robot_provi->blackbox_robot_EO.update_epoch_archi (); 
			
		  }
   }
   
}





param_eo eo1;

/**
 *
 * world::compute_fitness_per_cycle2()
 *
 * In this function, we will exactly compute the fitness given by the robot for only one cycle
 * 
 *
 */

Real compute_fitness_per_cycle2(world * w)
  
{
  
    return (*(eo1.compute_fitness_per_cycle))(w->vector_robot,*w->env);

}







/**
 *
 * compute_fitness()
 *
 * In this function, we will compute the fitness given by the robot
 *
 *
 * The fitness is computed in proceeding by this way:
 *
 * the robot is evaluated on several cycles (defined in the parser)
 *
 * this step is done x times (x representing the number of epoch, defined in the parser too)
 * 
 *
 */



Real compute_fitness(world * w)
{
  
  
  unsigned j;
  double fitness=0.0;
  for (unsigned i=0;i<eo1.number_of_epochs;i++)
    {      
	  
      new_epoch=true;

      look_archi(w);
      
      bool next_step=false;
	  	  	  
      for (j=0;j<w->vector_robot.size();j++)
	{
	  
	  if (w->vector_robot[j]->type()=="khepera_EO") {
#ifdef _WIN32
		khepera_EO * robot_provi =(static_cast<khepera_EO *>( w->vector_robot[j]));
#else
		khepera_EO * robot_provi =(dynamic_cast<khepera_EO *>( w->vector_robot[j]));
#endif

	  
	  if (robot_provi->set_pos)
	    {
	      
	      next_step = true;
	
     
	      w->vector_robot[j]->centerx=((robot_provi->vector_pos_robot)[robot_provi->step_epoch]).centerx;
 
	      w->vector_robot[j]->centery=((robot_provi->vector_pos_robot)[robot_provi->step_epoch]).centery;
 
	      w->vector_robot[j]->direction=((robot_provi->vector_pos_robot)[robot_provi->step_epoch]).direction;
 
	
	    }
	
	  else w->vector_robot[j]->choose_a_place(w->env->dx,w->env->dy); // we are in the random case

	  if (next_step) robot_provi->step_epoch++;
	
	  if (robot_provi->step_epoch == robot_provi->step_epoch_MAX) robot_provi->step_epoch=0;
	
	  
	  
	  }
	}
	  
	  for (j=0;j<eo1.number_of_cycles;j++)
	{
	  w->evolue();
	  fitness+= compute_fitness_per_cycle2(w);
	  if (at_least_one_crash) break;
	}
	  
	  
	  
	     
    }
  return fitness;  

}






//We prepare a function to manage keys hitted during simulation
// c is the char received
// (x,y) the position of the mouse in the window
void my_key_func(unsigned char c, int x, int y)
{
  switch(c) 
	{
	case 'e':case'E' :freeze=true; eo_activated=true;arret_demande=false; signal( SIGINT, signal_handler );
    signal( SIGQUIT, signal_handler ); break;

	case '=': world_GL.reset_robots_positions();
	new_epoch=true;
	break;

	case 'x' : case 'X' : robot_display++;if (robot_display>=int((*pop).size())) {robot_display=(*pop).size()-1;}
    cout << "Now displaying individual " <<robot_display<<endl;break;

	case 'c' : case 'C' : robot_display--;if (robot_display<0) {robot_display=0;}
    cout << "Now displaying individual " <<robot_display<<endl; break;
  
	}
}



void my_pre_idle_func()
{
  if (eo_activated)
	  {
		
		if ((*checkpoint)((*pop)))
			  {
				cout << "Entering evolution loop" << endl;
				

				// try{
				run_ea((*ga),(*pop)); // run the ga
	
				// }

				// catch (exception& e)
				// {
				// cout << e.what() << endl;
				// }
				
				
			  }
		
        
	  }

  eo_activated=false;
  
  // Indi Best = pop[0];
  //world_GL=world_eo;

  if (draw_eo)
	  {
		
		for (unsigned i=0;i<(*pop)[robot_display].number_of_robots;i++)
		  {
			if (world_GL.vector_robot[i]->type()=="khepera_EO") {
#ifdef _WIN32
		khepera_EO * robot_provi =(static_cast<khepera_EO *>( world_GL.vector_robot[i]));
#else
		khepera_EO * robot_provi =(dynamic_cast<khepera_EO *>( world_GL.vector_robot[i]));
#endif
		
			robot_provi->update_weights((*pop)[robot_display].robots[i].poids);
			}
		  }
		
		//  world_GL.compute_fitness_per_cycle2();
	  }
  

}


void my_post_idle_func()
{
  // Ce truc tait dans une fonction de dessin (drawFunc) !!!! pkoi ?????
   if (draw_eo)
	{
	 
   compute_fitness_per_cycle2(&world_GL);
	
	}
}





/**
 *
 * altern_position(robot * robot_provi)
 *
 * In this function, we will change the position of the robot, in general the list of positions are 
 * initialised in the parser and this function is called at the beginning of each epoch
 *
 *
 */

void altern_position(robot * _robot_provi)
{

  if (_robot_provi->type()=="khepera_EO") {
	
  
#ifdef _WIN32
    khepera_EO * robot_provi =(static_cast<khepera_EO *>( _robot_provi));
#else
    khepera_EO * robot_provi =(dynamic_cast<khepera_EO *>( _robot_provi));
#endif

	if ((robot_provi)->set_pos)
	  {
		robot_provi->step_epoch++;

		if (robot_provi->step_epoch == robot_provi->step_epoch_MAX) robot_provi->step_epoch=0;	
		
		robot_provi->centerx=((robot_provi->vector_pos_robot)[robot_provi->step_epoch]).centerx;
		
		robot_provi->centery=((robot_provi->vector_pos_robot)[robot_provi->step_epoch]).centery;
  
		robot_provi->direction=((robot_provi->vector_pos_robot)[robot_provi->step_epoch]).direction;
	  } else {
		
		(_robot_provi)->choose_a_place();
		
	  }
  }

  else {
	(_robot_provi)->choose_a_place();
	
  }
  
  
	
}







int main (int argc, char *argv[]) // int argc, char *argv[ ], char *envp[ ] )
{
     
  //first of all : we think the name of function is expressive enough	 
  if (GL_world_initialisation(argc,argv)==1) {return 1; }//error with OpenGL
  
  //We register our callback function for EO keys management (Optionnal)
  Register_my_keyfunc(my_key_func);

  //We register our pre_idle_func (she will be called at each time step before the evolution)
  Register_my_pre_idle_func(my_pre_idle_func);

  //We register our pre_idle_func (she will be called at each time step after the evolution)
  Register_my_post_idle_func(my_post_idle_func);

  
  //secondly, we add the parameters for the environment and for the robots
  try
	
	{

	 
      srand(time(NULL)); //random generator initialisation
	  
      environment env0(WIDTH,HEIGHT);
      
      eoParser parser(argc, argv);
      eoState state;

      check_param_environment (parser,env0);
      

      world world_eo; 
      world_eo.set_environment(env0);

      //world_GL=world_eo;

      world_GL.set_environment(env0);

	  //Set the special restart function
	  world_GL.Register_my_restart_func(altern_position);
	  
	  
	  //prepare the controller list :
	  Controller *  ctrl=new Controller (controller_with_blackbox);
	  all_the_controllers.add_a_controller("controller_with_blackbox",ctrl);
	  
	  
	  all_the_controllers.add_a_controller("controller_save",new Controller (controller_save));
	  
	  all_the_controllers.add_a_controller("simple_supervisor",new Controller (simple_supervisor));

	  all_the_controllers.add_a_controller("symbolic_controller",new Controller (symbolic_controller));

	  all_the_controllers.add_a_controller("controller_battery",new Controller (controller_battery));


      map_fitnesses all_fitnesses_available; //Activate all the fitnesses

      check_param_problem(all_fitnesses_available,world_eo,parser);


      eoRNInit<Indi>& init = make_genotype(parser, state, world_eo); 
      // Build the variation operator (any seq/prop construct)
   
      eoEvalSimulation<Indi> eval(world_eo);

      eoEvalFuncCounter<Indi> eval_all_the_robots (eval);

      eoGenOp<Indi>& op = make_op(parser, state, init);


      //// Now the representation-independent things 
      //
      // YOU SHOULD NOT NEED TO MODIFY ANYTHING BEYOND THIS POINT
      // unless you want to add specific statistics to the checkpoint
      //////////////////////////////////////////////

      // initialize the population
      // yes, this is representation indepedent once you have an eoInit
      pop   = & make_pop(parser, state, init);
      // stopping criteria
      eoContinue<Indi> & term = make_continue(parser, state, eval_all_the_robots);
      // output

      //(*checkpoint)

      eoCheckPoint<Indi> & check = make_checkpoint(parser, state, eval_all_the_robots, term);


      checkpoint = & check;

          // algorithm (need the operator!)
      ga = & make_algo_scalar(parser, state, eval_all_the_robots, (*checkpoint), op);

                  /////////////////////////////////////////
      // to be called AFTER all parameters have been read!!!

        make_help(parser);
      
      ///////
      // evaluate intial population AFTER help and status in case it takes time
      apply(eval_all_the_robots, (*pop));
      
      eo_activated=true;
	  draw_eo=true;
	  
	  
      // boucle principale de gestion des vnements (only if you whish to use OPENG_GL display)
      glutMainLoop();
	  
      return 0;
	  
		
	}
  
 
  catch (error & _error)
	
	{
	  _error.what();
	}
  
  return 0;
  
  }
  
#else
#include <stdio.h>
int main()
{
  std::cout<<"You have defined DONT_USE_EO in src/heart/global.h ; This excecutable can't work since it uses intensivly the EO library, you have to edit global.h and to recompile"<<endl;
  
  return 0;
}

#endif


